Package: ExperimentHub
Type: Package
Title: Client to access ExperimentHub resources
Version: 3.2.0
Authors@R: 
    c(person("Bioconductor Package", "Maintainer", email="maintainer@bioconductor.org",
             role="cre"),
      person("Martin", "Morgan", role="aut"),
      person("Marc", "Carlson", role="ctb"),
      person("Dan", "Tenenbaum", role="ctb"),
      person("Sonali", "Arora", role="ctb"),
      person("Valerie", "Oberchain", role="ctb"),
      person("Kayla", "Morrell", role="ctb"),
      person("Lori", "Shepherd", role="aut"))
Description: This package provides a client for the Bioconductor
        ExperimentHub web resource. ExperimentHub provides a central location
        where curated data from experiments, publications or training courses
        can be accessed. Each resource has associated metadata, tags and date
        of modification. The client creates and manages a local cache of files
        retrieved enabling quick and reproducible access.
License: Artistic-2.0
biocViews: Infrastructure, DataImport, GUI, ThirdPartyClient
Depends: methods, BiocGenerics (>= 0.15.10), AnnotationHub (>= 3.99.3),
        BiocFileCache (>= 2.99.3)
Imports: utils, S4Vectors, BiocManager, rappdirs
Suggests: knitr, BiocStyle, rmarkdown, HubPub, GenomicAlignments
Enhances: ExperimentHubData
URL: https://github.com/Bioconductor/ExperimentHub
BugReports: https://github.com/Bioconductor/ExperimentHub/issues
VignetteBuilder: knitr
RoxygenNote: 7.0.0
git_url: https://git.bioconductor.org/packages/ExperimentHub
git_branch: RELEASE_3_23
git_last_commit: 2b9e0cc
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:07:55 UTC; biocbuild
Author: Bioconductor Package Maintainer [cre],
  Martin Morgan [aut],
  Marc Carlson [ctb],
  Dan Tenenbaum [ctb],
  Sonali Arora [ctb],
  Valerie Oberchain [ctb],
  Kayla Morrell [ctb],
  Lori Shepherd [aut]
Maintainer: Bioconductor Package Maintainer <maintainer@bioconductor.org>
Built: R 4.6.0; ; 2026-04-29 04:42:27 UTC; unix

Package: ExperimentHub
Type: Package
Title: Client to access ExperimentHub resources
Version: 3.2.2
Authors@R: 
    c(person("Bioconductor Package", "Maintainer", email="maintainer@bioconductor.org",
             role="cre"),
      person("Martin", "Morgan", role="aut"),
      person("Marc", "Carlson", role="ctb"),
      person("Dan", "Tenenbaum", role="ctb"),
      person("Sonali", "Arora", role="ctb"),
      person("Valerie", "Oberchain", role="ctb"),
      person("Kayla", "Morrell", role="ctb"),
      person("Lori", "Shepherd", role="aut"))
Description: This package provides a client for the Bioconductor
        ExperimentHub web resource. ExperimentHub provides a central location
        where curated data from experiments, publications or training courses
        can be accessed. Each resource has associated metadata, tags and date
        of modification. The client creates and manages a local cache of files
        retrieved enabling quick and reproducible access.
License: Artistic-2.0
biocViews: Infrastructure, DataImport, GUI, ThirdPartyClient
Depends: methods, BiocGenerics (>= 0.15.10), AnnotationHub (>= 3.99.3),
        BiocFileCache (>= 2.99.3)
Imports: utils, S4Vectors, BiocManager, rappdirs
Suggests: knitr, BiocStyle, rmarkdown, HubPub, GenomicAlignments,
        SingleCellExperiment
Enhances: ExperimentHubData
URL: https://github.com/Bioconductor/ExperimentHub
BugReports: https://github.com/Bioconductor/ExperimentHub/issues
VignetteBuilder: knitr
RoxygenNote: 7.0.0
git_url: https://git.bioconductor.org/packages/ExperimentHub
git_branch: RELEASE_3_23
git_last_commit: 9ab42ac
git_last_commit_date: 2026-08-19
Repository: Bioconductor 3.23
Date/Publication: 2026-08-19
NeedsCompilation: no
Packaged: 2026-08-19 21:54:32 UTC; biocbuild
Author: Bioconductor Package Maintainer [cre],
  Martin Morgan [aut],
  Marc Carlson [ctb],
  Dan Tenenbaum [ctb],
  Sonali Arora [ctb],
  Valerie Oberchain [ctb],
  Kayla Morrell [ctb],
  Lori Shepherd [aut]
Maintainer: Bioconductor Package Maintainer <maintainer@bioconductor.org>

import(methods)
import(BiocGenerics)
import(S4Vectors)
import(AnnotationHub)
importFrom(BiocManager, install)
import(BiocFileCache)
importFrom(tools, R_user_dir)
importFrom(rappdirs, user_cache_dir)

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Export S4 classes
###

exportClasses(ExperimentHub) 

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Export S4 methods for generics not defined in AnnotationHub 
###

exportMethods(
    cache, "[[", 
    listResources, loadResources, package
)

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Export S4 generics defined in AnnotationHub + export corresponding
### methods
###

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Export non-generic functions
###

export(ExperimentHub, 
       getExperimentHubOption, setExperimentHubOption,
       createHubAccessors
)

.CACHE_ROOT <- "ExperimentHub"

.onLoad <- function(libname, pkgname, ...) {
    ## options from getOption or Sys.env or default, in that order
    if (is.null(getExperimentHubOption("MAX_DOWNLOADS"))) {
        opt <- getOption("EXPERIMENT_HUB_MAX_DOWNLOADS", 10L)
        opt <- Sys.getenv("EXPERIMENT_HUB_MAX_DOWNLOADS", opt)
        opt <- as.integer(opt)
        setExperimentHubOption("MAX_DOWNLOADS", opt)
    }
    if (is.null(getExperimentHubOption("URL"))) {
        opt <- getOption("EXPERIMENT_HUB_URL",
                         "https://experimenthub.bioconductor.org")
        opt <- Sys.getenv("EXPERIMENT_HUB_URL", opt)
        setExperimentHubOption("URL", opt)
    }
    if (is.null(getExperimentHubOption("CACHE"))) {
        path <- tools::R_user_dir(.CACHE_ROOT, which="cache")
        opt <- getOption("EXPERIMENT_HUB_CACHE", path)
        opt <- Sys.getenv("EXPERIMENT_HUB_CACHE", opt)
        setExperimentHubOption("CACHE", opt)
    }
    if (is.null(getExperimentHubOption("PROXY"))) {
        opt <- getOption("EXPERIMENT_HUB_PROXY", "")
        opt <- Sys.getenv("EXPERIMENT_HUB_PROXY", opt)
        if (nzchar(opt))
            setExperimentHubOption("PROXY", opt)
    }
    if (is.null(getExperimentHubOption("LOCAL"))) {
        opt <- getOption("EXPERIMENT_HUB_LOCAL", FALSE)
        opt <- Sys.getenv("EXPERIMENT_HUB_LOCAL", opt)
        opt <- as.logical(opt)
        setExperimentHubOption("LOCAL", opt)
    }
    if (is.null(getExperimentHubOption("ASK"))) {
        opt <- getOption("EXPERIMENT_HUB_ASK", interactive())
        opt <- Sys.getenv("EXPERIMENT_HUB_ASK", opt)
        opt <- as.logical(opt)
        setExperimentHubOption("ASK", opt)
    }

}

.EH_hub_options <- new.env(parent=emptyenv())

getExperimentHubOption <- function(arg) {
    stopifnot(is.character(arg), length(arg) == 1L)
    arg <- toupper(arg)
    key <- c("URL", "CACHE", "PROXY", "MAX_DOWNLOADS", "LOCAL", "ASK")
    if (!arg %in% key)
        stop("'arg' must be one of 'URL', 'CACHE', 'PROXY', ",
             "'LOCAL', 'ASK' or 'MAXDOWNLOADS'")
    .EH_hub_options[[arg]]
}

setExperimentHubOption <- function(arg, value)
{
    stopifnot(is.character(arg), length(arg) == 1L)
    key <- .hub_option_key(toupper(trimws(arg)))

    .EH_hub_options[[key]] <- switch(key, URL=, CACHE={
        value <- as.character(value)
        stopifnot(isSingleString(value))
        value
    }, MAX_DOWNLOADS={
        value <- as.integer(value)
        stopifnot(isSingleInteger(value))
        value
    }, PROXY={
        if (is.null(value) || (isSingleString(value)))
            value
        else {
            txt <-
                "'value' must be an single string value character(1) in embedded format (e.g. http://my_user:my_password@myproxy:8080), or NULL"
            stop(paste(strwrap(txt, exdent=2), collapse="\n"))
        }
    }, LOCAL={
        stopifnot(isTRUE(value) | isFALSE(value))
        value
    }, ASK={
        stopifnot(isTRUE(value) || isFALSE(value))
        value
    })
}

.EH_hub_options <- new.env(parent=emptyenv())

getExperimentHubOption <- function(arg) {
    stopifnot(is.character(arg), length(arg) == 1L)
    arg <- toupper(arg)
    key <- c("URL", "CACHE", "PROXY", "MAX_DOWNLOADS", "LOCAL", "ASK")
    if (!arg %in% key)
        stop("'arg' must be one of 'URL', 'CACHE', 'PROXY', ",
             "'LOCAL', 'ASK' or 'MAXDOWNLOADS'")
    .EH_hub_options[[arg]]
}

setExperimentHubOption <- function(arg, value)
{
    stopifnot(is.character(arg), length(arg) == 1L)
    key <- .hub_option_key(toupper(trimws(arg)))

    .EH_hub_options[[key]] <- switch(key, URL=, CACHE={
        value <- as.character(value)
        stopifnot(isSingleString(value))
        value
    }, MAX_DOWNLOADS={
        value <- as.integer(value)
        stopifnot(isSingleInteger(value))
        value
    }, PROXY={
        if (is.null(value) | inherits(value, "request"))
            value
        else if (isSingleString(value)) {
            .httr_proxy(value)
        } else {
            txt <- "'value' must be an httr proxy request (use_proxy()),
                    character(1), or NULL"
            stop(paste(strwrap(txt, exdent=2), collapse="\n"))
        }
    }, LOCAL={
        stopifnot(isTRUE(value) | isFALSE(value))
        value
    }, ASK={
        stopifnot(isTRUE(value) || isFALSE(value))
        value
    })
}

### =========================================================================
### ExperimentHub objects
### -------------------------------------------------------------------------
###

setClass("ExperimentHub", contains="Hub")

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Constructor
###

ExperimentHub <-
    function(..., hub=getExperimentHubOption("URL"),
             cache=getExperimentHubOption("CACHE"),
             proxy=getExperimentHubOption("PROXY"),
             localHub=getExperimentHubOption("LOCAL"),
             ask=getExperimentHubOption("ASK"))
{
    if ((cache == R_user_dir("ExperimentHub", which="cache")) && (Sys.getenv("EXPERIMENT_HUB_CACHE")=="")){
        olddefault = rappdirs::user_cache_dir(appname="ExperimentHub")
        if (dir.exists(olddefault) && (length(list.files(olddefault)) != 0)){
            stop(msg=paste0("DEFUNCT: As of ExperimentHub (>1.17.2), default caching location has changed.\n",
                 "  Problematic cache: ", path.expand(olddefault), "\n",
                 "  See https://bioconductor.org/packages/devel/bioc/vignettes/ExperimentHub/inst/doc/ExperimentHub.html#default-caching-location-update\n"))
            cache = olddefault
        }
    }

    if (is.null(proxy)){
        connect <- suppressWarnings(tryCatch({
            readBin(hub, n=1L, what="raw")
            TRUE
        }, error = function(...){
            FALSE
        }))
    } else {
        connect <- TRUE
        message("Assuming valid proxy connection through '",
                ifelse(is(proxy,"request"),
                       paste(unlist(proxy), collapse=":"),
                       proxy),
                "'",
                "\n If you experience connection issues consider ",
                "using 'localHub=TRUE'")
    }
    if (!connect && !localHub){
        message("Cannot connect to ExperimentHub server, using 'localHub=TRUE' instead")
        localHub <- !connect
    }
    if(localHub){
        message("Using 'localHub=TRUE'\n",
                "  If offline, please also see BiocManager vignette section on offline use")
    }
    .Hub("ExperimentHub", hub, cache, proxy, localHub, ask, ...)
}

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Accessor-like methods
###

setMethod("package", "ExperimentHub",
    function(x)
{
    query <- sprintf("SELECT preparerclass FROM resources\n
                     WHERE resources.id IN (%s)",
             AnnotationHub:::.id_as_single_string(x))
    ans <- AnnotationHub:::.db_query(dbfile(x), query)[["preparerclass"]]
    names(ans) <- names(x)
    ans
})

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### cache methods
###

setMethod("cache", "ExperimentHub",
    function(x, ..., force=FALSE, verbose=FALSE) {
        callNextMethod(x,
                       cache.root="ExperimentHub",
                       cache.fun=setExperimentHubOption,
                       proxy=getExperimentHubOption("PROXY"),
                       max.downloads=getExperimentHubOption("MAX_DOWNLOADS"),
                       force=force, verbose=verbose)
    }
)

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Subsetting
###

.tryload <- function(pkg) {
    ## Try to install/load; don't fail if not available
    success <- paste0("see ?", pkg, " and browseVignettes('",
                      pkg, "') for documentation")
    if (!pkg %in% rownames(installed.packages()))
        BiocManager::install(pkg, suppressUpdates=TRUE)
    if (pkg %in% rownames(installed.packages())) {
        suppressPackageStartupMessages({
            require(pkg, quietly = TRUE, character.only = TRUE)
        })
        message(success)
    }
}

setMethod("[[", c("ExperimentHub", "numeric", "missing"),
    function(x, i, j, ..., force=FALSE, verbose=TRUE)
{
    if (length(x[i]) != 1L)
        stop("'i' must be length 1")
    pkg <- AnnotationHub:::.count_resources(x[i], "preparerclass")
    .tryload(pkg)
    callNextMethod(x, i, j, ..., force=force, verbose=verbose)
    ## or AnnotationHub:::.Hub_get1(x[i])
})

setMethod("[[", c("ExperimentHub", "character", "missing"),
    function(x, i, j, ..., force=FALSE, verbose=TRUE)
{
    if (length(i) != 1L)
        stop("'i' must be length 1")
    idx <- match(i, names(AnnotationHub:::.db_uid(x)))
    if (is.na(idx)){
        status = recordStatus(x, i)
        if (isLocalHub(x)){
            if (status$status == "Public")
                stop(
                    "File not previously downloaded.\n",
                    "  Run with 'localHub=FALSE'",
                    call.=FALSE
                )
        }
        if (status$status == "Public" && (as.Date(status$dateadded) >=
                as.Date(snapshotDate(x))))
            stop(status$record, " added after current Hub snapshot date.\n",
                 "  added: ", as.character(status$dateadded), "\n",
                 "  snapshote date: ",  as.character(snapshotDate(x)),
                 call.=FALSE)

        msg <- status$status
        if (!is.null(status$dateremoved))
            msg <- paste0(msg, "\n  Resource removed on: ", status$dateremoved)
        stop(msg, call.=FALSE)
    }

    pkg <- AnnotationHub:::.count_resources(x[i], "preparerclass")
    .tryload(pkg)
    callNextMethod(x, i, j, ..., force=force, verbose=verbose)
    ## or AnnotationHub:::.Hub_get1(x[idx])
})

### =========================================================================
### ExperimentHub objects
### -------------------------------------------------------------------------
###

setClass("ExperimentHub", contains="Hub")

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Constructor
###

ExperimentHub <-
    function(..., hub=getExperimentHubOption("URL"),
             cache=getExperimentHubOption("CACHE"),
             proxy=getExperimentHubOption("PROXY"),
             localHub=getExperimentHubOption("LOCAL"),
             ask=getExperimentHubOption("ASK"))
{
    if ((cache == R_user_dir("ExperimentHub", which="cache")) && (Sys.getenv("EXPERIMENT_HUB_CACHE")=="")){
        olddefault = rappdirs::user_cache_dir(appname="ExperimentHub")
        if (dir.exists(olddefault) && (length(list.files(olddefault)) != 0)){
            stop(msg=paste0("DEFUNCT: As of ExperimentHub (>1.17.2), default caching location has changed.\n",
                 "  Problematic cache: ", path.expand(olddefault), "\n",
                 "  See https://bioconductor.org/packages/devel/bioc/vignettes/ExperimentHub/inst/doc/ExperimentHub.html#default-caching-location-update\n"))
            cache = olddefault
        }
    }

    if (!is.null(proxy)) {
        message("Assuming valid proxy connection through '",
                ifelse(is(proxy,"request"),
                       paste(unlist(proxy), collapse=":"),
                       proxy),
                "'",
                "\n If you experience connection issues consider ",
                "using 'localHub=TRUE'")
    } else if (!localHub) {
        connect <- suppressWarnings(tryCatch({
            readBin(hub, n=1L, what="raw")
            TRUE
        }, error = function(...){
            FALSE
        }))
        if (!connect){
            message("Cannot connect to ExperimentHub server, using 'localHub=TRUE' instead")
            localHub <- FALSE
        }
    }
    if(localHub){
        message("Using 'localHub=TRUE'\n",
                "  If offline, please also see BiocManager vignette section on offline use")
    }
    .Hub("ExperimentHub", hub, cache, proxy, localHub, ask, ...)
}

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Accessor-like methods
###

setMethod("package", "ExperimentHub",
    function(x)
{
    query <- sprintf("SELECT preparerclass FROM resources\n
                     WHERE resources.id IN (%s)",
             AnnotationHub:::.id_as_single_string(x))
    ans <- AnnotationHub:::.db_query(dbfile(x), query)[["preparerclass"]]
    names(ans) <- names(x)
    ans
})

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### cache methods
###

setMethod("cache", "ExperimentHub",
    function(x, ..., force=FALSE, verbose=FALSE, config=list(), progress=TRUE) {
        callNextMethod(x,
                       cache.root="ExperimentHub",
                       cache.fun=setExperimentHubOption,
                       proxy=getExperimentHubOption("PROXY"),
                       max.downloads=getExperimentHubOption("MAX_DOWNLOADS"),
                       force=force, verbose=verbose, config=config, progress=progress)
    }
)

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Subsetting
###

.tryload <- function(pkg) {
    ## Try to install/load; don't fail if not available
    success <- paste0("see ?", pkg, " and browseVignettes('",
                      pkg, "') for documentation")
    nopkg <- paste0(pkg,
                    " not installed.\n  Full functionality, documentation, ",
                    "and loading of data might not be possible without installing")
    if (!requireNamespace(pkg, quietly = TRUE)){
        message(nopkg)
        if (interactive() &&
            identical(AnnotationHub:::.ask(
                                          paste0(txt="Install ", pkg),
                                          values=c("yes", "no")),
                      "yes")
            ){
            BiocManager::install(pkg, update=FALSE)
        }
    }
    if (requireNamespace(pkg, quietly = TRUE)) {
        suppressPackageStartupMessages({
            require(pkg, quietly = TRUE, character.only = TRUE)
        })
        message(success)
    }
}

setMethod("[[", c("ExperimentHub", "numeric", "missing"),
    function(x, i, j, ..., force=FALSE, verbose=TRUE, config=list(), progress=TRUE)
{
    if (length(x[i]) != 1L)
        stop("'i' must be length 1")
    pkg <- AnnotationHub:::.count_resources(x[i], "preparerclass")
    .tryload(pkg)
    callNextMethod(x, i, j, ..., force=force, verbose=verbose, config=config, progress=progress)
    ## or AnnotationHub:::.Hub_get1(x[i])
})

setMethod("[[", c("ExperimentHub", "character", "missing"),
    function(x, i, j, ..., force=FALSE, verbose=TRUE, config=list(), progress=TRUE)
{
    if (length(i) != 1L)
        stop("'i' must be length 1")
    idx <- match(i, names(AnnotationHub:::.db_uid(x)))
    if (is.na(idx)){
        status = recordStatus(x, i)
        if (isLocalHub(x)){
            if (status$status == "Public")
                stop(
                    "File not previously downloaded.\n",
                    "  Run with 'localHub=FALSE'",
                    call.=FALSE
                )
        }
        if (status$status == "Public" && (as.Date(status$dateadded) >=
                as.Date(snapshotDate(x))))
            stop(status$record, " added after current Hub snapshot date.\n",
                 "  added: ", as.character(status$dateadded), "\n",
                 "  snapshote date: ",  as.character(snapshotDate(x)),
                 call.=FALSE)

        msg <- status$status
        if (!is.null(status$dateremoved))
            msg <- paste0(msg, "\n  Resource removed on: ", status$dateremoved)
        stop(msg, call.=FALSE)
    }

    pkg <- AnnotationHub:::.count_resources(x[i], "preparerclass")
    .tryload(pkg)
    callNextMethod(x, i, j, ..., force=force, verbose=verbose, config=config, progress=progress)
    ## or AnnotationHub:::.Hub_get1(x[idx])
})

### =========================================================================
### Helpers for package-specific resource discovery
### -------------------------------------------------------------------------

## export resources as accessor functions
.HUB <- new.env(parent = emptyenv())

.GET_HUB <- function() get("eh", envir = .HUB)

.SET_HUB <- function(value) assign("eh", value, envir=.HUB)

.get_ExperimentHub <- function() {
     eh <- try(.GET_HUB(), silent=TRUE)
     if (inherits(eh, "try-error")) {
       eh <- ExperimentHub::ExperimentHub()
       .SET_HUB(eh)
     }
     eh
}

.hubAccessorFactory <- function(pkgname, title) {
    force(title)
    function(metadata=FALSE) {
        ehub <- .get_ExperimentHub()
        eh1 <- ehub[package(ehub) == pkgname & ehub$title == title]
        if (length(eh1) == 0L)
            stop("\"", title, "\" not found in ExperimentHub")
        if (length(eh1) != 1L)
            stop("\"", title, "\" matches more than 1 ExperimentHub resource")
        if (metadata) {
            eh1
        } else
            eh1[[1L]]
    }
}

createHubAccessors <- function(pkgname, titles) {
    ## Create and export accessor functions in package namespace.
    ns <- asNamespace(pkgname)
    for (title in titles) {
        assign(title, .hubAccessorFactory(pkgname, title), envir=ns)
        namespaceExport(ns, title)
    }
}

## resource discovery

.filterResources_EH <- function(package, filterBy=character()) {
    if (!is.character(filterBy))
        stop("'filterBy' must be a character vector")
    suppressMessages({eh <- ExperimentHub()})
    if (!package %in% unique(package(eh)))
        stop(paste0("'", package, "' resources were not found in ExperimentHub"))

    sub <- query(eh, package)
    if (length(filterBy))
        query(sub, filterBy)
    else
        sub
}

setMethod("listResources", "ExperimentHub",
    function(hub, package, filterBy=character()) {
        metadata <- .filterResources_EH(package, filterBy)
        mcols(metadata)$title
})

setMethod("loadResources", "ExperimentHub",
    function(hub, package, filterBy=character()) {
        metadata <- .filterResources_EH(package, filterBy)
        eh <- ExperimentHub()
        lapply(names(metadata), function(i) eh[[i]])
})

### =========================================================================
### Helpers for package-specific resource discovery
### -------------------------------------------------------------------------

## export resources as accessor functions
.HUB <- new.env(parent = emptyenv())

.GET_HUB <- function() get("eh", envir = .HUB)

.SET_HUB <- function(value) assign("eh", value, envir=.HUB)

.get_ExperimentHub <- function() {
     eh <- try(.GET_HUB(), silent=TRUE)
     if (inherits(eh, "try-error")) {
       eh <- ExperimentHub::ExperimentHub()
       .SET_HUB(eh)
     }
     eh
}

.hubAccessorFactory <- function(ehid) {
     force(ehid)
     function(metadata=FALSE) {
         eh <- .get_ExperimentHub()
         if (metadata) {
             eh[ehid]
         } else
             eh[[ehid]]
     }
}

createHubAccessors <- function(pkgname, titles) {
    ## map titles to ExperimentHub identifiers
    eh <- query(.get_ExperimentHub(), pkgname)

    ehids <- vapply(titles, function(tle, exactMatch) {
        ehid <- names(subset(eh, eh$title == tle))
        if (length(ehid) == 0L) {
            stop(sQuote(tle), " not found in ExperimentHub")
        } else if (length(ehid) != 1L) {
            stop(sQuote(tle),
                 " matches more than 1 ExperimentHub resource")
        }
        ehid
    }, character(1))

    ## create and export accessor functions in package namespace
    ns <- asNamespace(pkgname)
    for (i in seq_along(titles)) {
        assign(titles[[i]], .hubAccessorFactory(ehids[[i]]), envir=ns)
        namespaceExport(ns, titles[[i]])
    }
}

## resource discovery

.filterResources_EH <- function(package, filterBy=character()) {
    if (!is.character(filterBy))
        stop("'filterBy' must be a character vector")
    suppressMessages({eh <- ExperimentHub()})
    if (!package %in% unique(package(eh)))
        stop(paste0("'", package, "' resources were not found in ExperimentHub"))

    sub <- query(eh, package)
    if (length(filterBy))
        query(sub, filterBy)
    else
        sub
}

setMethod("listResources", "ExperimentHub",
    function(hub, package, filterBy=character()) {
        metadata <- .filterResources_EH(package, filterBy)
        mcols(metadata)$title
})

setMethod("loadResources", "ExperimentHub",
    function(hub, package, filterBy=character()) {
        metadata <- .filterResources_EH(package, filterBy)
        eh <- ExperimentHub()
        lapply(names(metadata), function(i) eh[[i]])
})

##
##  Start by investigating what we list as "Public" available but no
##  rdatadateremoved 
##

load("ehresults.RData")
dim(eh_bad)
## 20 5
eh_bad
## 1-6 https://recount.bio/data/ SSL certificate error (asked recountmethylation
## package maintainer to reach out to webmaster of site)
##
## 7-8 case sensitivity issue -- renames on Azure
##
## 9-16 https://auckland.figshare.com these seem to be available when manually
## navigated too manually. Further investigation needed? homosapienDEE2CellScore
## false positive? maybe HEAD not activated on this server?
##
## 17-20 -- leukemiaAtlas package upload was a mess, seems wrong and may have to
## delete and reupload/add to database
## 




load("ahresults.RData")
## 12487     5
ah_bad

prefix <- unique(ah_bad$location_prefix)
prefix
## [1] "https://bioconductorhubs.blob.core.windows.net/annotationhub/"
## [2] "http://amp.pharm.mssm.edu/"                                   
## [3] "http://www.pazar.info/"                                       
## [4] "http://haemcode.stemcells.cam.ac.uk/"                         
## [5] "http://egg2.wustl.edu/roadmap/data/byFileType/"               
## [6] "ftp://ftp.pride.ebi.ac.uk/"                                   
## [7] "ftp://ftp.ebi.ac.uk/pub/databases/gencode/"                   
## [8] "ftp://ftp.ensembl.org/pub/"                                   
## [9] "ftp://ftp.ncbi.nlm.nih.gov/"

for(i in prefix){
    message(i, ": ", length(which(ah_bad$location_prefix == i)))
}
## https://bioconductorhubs.blob.core.windows.net/annotationhub/: 61
## http://amp.pharm.mssm.edu/: 1
## http://www.pazar.info/: 91
## http://haemcode.stemcells.cam.ac.uk/: 16
## http://egg2.wustl.edu/roadmap/data/byFileType/: 39
## ftp://ftp.pride.ebi.ac.uk/: 1
## ftp://ftp.ebi.ac.uk/pub/databases/gencode/: 31
## ftp://ftp.ensembl.org/pub/: 12231
## ftp://ftp.ncbi.nlm.nih.gov/: 16


"http://amp.pharm.mssm.edu/"
## 1 file
## unreachable 404 :  recommend removing AH22237 : listed maintainer is
## Bioconductor Maintainer

"ftp://ftp.pride.ebi.ac.uk/"
## 1 file 
## Other files at this endpoint are okay. AH49009 : Laurent Gatto maintainer and
## mentioned in ProteomicsAnnotationHubData that was deprecated in
## 3.16. contacted

"http://haemcode.stemcells.cam.ac.uk/"
## 16 files
## HAEMCODE has moved!! Redirected "https://codex.stemcells.cam.ac.uk/"
## HaemCodeImportPreparer, Bioconductor Maintainer
## But there are 945 haemcode references in the hub, only these 16 are not
## accessible? "AH27635"-"AH28579". All 945 use the same location_prefix listed
## above.
## Bad Ids:
##    AH28151, AH28152, AH28153, AH28154, AH28155,
##    AH28257,
##    AH28262, AH28263,
##    AH28466, AH28467, AH28468, AH28469, AH28470,
##    AH28572,
##    AH28577, AH28578
##    

"ftp://ftp.ncbi.nlm.nih.gov/"
## 16 files
## dbSNPVCFPreparer, Bioconductor Maintainer
## Corresponds to 8 records because they are vcf.gz and vcf.gz.tbi
## AH57956-AH57963  
## is it an issue with ftp://ftp as when manually redirect to https site
## https://ftp.ncbi.nlm.nih.gov/pub/clinvar/vcf_GRCh38/archive_1.0/2016/
## exists. Would switching to this base url interfer with load see
## AnnotationHubData:::ncbi_dbSNPVCFFile


"ftp://ftp.ebi.ac.uk/pub/databases/gencode/"
## 31 files
## GencodeGffImportPreparer, Bioconductor Maintainer
## AH49545-AH49553  recipe is blank
## AH49554-AH49562  recipe is blank
## AH75118-AH75126  recipe is AnnotationHubData:::gencodeGFFToGRanges
## AH75127-AH75130  recipe is AnnotationHubData:::gencodeGFFToGRanges
## Newer package GenomicScores added some repeat/newer data. Maybe these are not
## needed anymore?
## will changing from ftp://ftp to the available https break load recipe?
## https://ftp.ebi.ac.uk/pub/databases/gencode/Gencode_mouse/release_M6/
## https://ftp.ebi.ac.uk/pub/databases/gencode/Gencode_human/release_23/
## https://ftp.ebi.ac.uk/pub/databases/gencode/Gencode_human/release_31/
## https://ftp.ebi.ac.uk/pub/databases/gencode/Gencode_human/release_31/GRCh37_mapping/

"http://egg2.wustl.edu/roadmap/data/byFileType/"
## 39 files
## Many more at this endpoint available
## All these files are still available but have changed to a v2 only
## example: peaks/consolidated/narrowPeak/E003-DNase.hotspot.all.peaks.v2.bed.gz
## solution: update rdatapath manually in database for these records
ah_bad[which(ah_bad$location_prefix == "http://egg2.wustl.edu/roadmap/data/byFileType/"), "rdatapath"]
##  [1] "peaks/consolidated/narrowPeak/E003-DNase.hotspot.all.peaks.bed.gz"
##  [2] "peaks/consolidated/narrowPeak/E004-DNase.hotspot.all.peaks.bed.gz"
##  [3] "peaks/consolidated/narrowPeak/E005-DNase.hotspot.all.peaks.bed.gz"
##  [4] "peaks/consolidated/narrowPeak/E006-DNase.hotspot.all.peaks.bed.gz"
##  [5] "peaks/consolidated/narrowPeak/E007-DNase.hotspot.all.peaks.bed.gz"
##  [6] "peaks/consolidated/narrowPeak/E008-DNase.hotspot.all.peaks.bed.gz"
##  [7] "peaks/consolidated/narrowPeak/E017-DNase.hotspot.all.peaks.bed.gz"
##  [8] "peaks/consolidated/narrowPeak/E021-DNase.hotspot.all.peaks.bed.gz"
##  [9] "peaks/consolidated/narrowPeak/E022-DNase.hotspot.all.peaks.bed.gz"
## [10] "peaks/consolidated/narrowPeak/E028-DNase.hotspot.all.peaks.bed.gz"
## [11] "peaks/consolidated/narrowPeak/E032-DNase.hotspot.all.peaks.bed.gz"
## [12] "peaks/consolidated/narrowPeak/E033-DNase.hotspot.all.peaks.bed.gz"
## [13] "peaks/consolidated/narrowPeak/E050-DNase.hotspot.all.peaks.bed.gz"
## [14] "peaks/consolidated/narrowPeak/E051-DNase.hotspot.all.peaks.bed.gz"
## [15] "peaks/consolidated/narrowPeak/E055-DNase.hotspot.all.peaks.bed.gz"
## [16] "peaks/consolidated/narrowPeak/E056-DNase.hotspot.all.peaks.bed.gz"
## [17] "peaks/consolidated/narrowPeak/E057-DNase.hotspot.all.peaks.bed.gz"
## [18] "peaks/consolidated/narrowPeak/E059-DNase.hotspot.all.peaks.bed.gz"
## [19] "peaks/consolidated/narrowPeak/E080-DNase.hotspot.all.peaks.bed.gz"
## [20] "peaks/consolidated/narrowPeak/E081-DNase.hotspot.all.peaks.bed.gz"
## [21] "peaks/consolidated/narrowPeak/E082-DNase.hotspot.all.peaks.bed.gz"
## [22] "peaks/consolidated/narrowPeak/E083-DNase.hotspot.all.peaks.bed.gz"
## [23] "peaks/consolidated/narrowPeak/E084-DNase.hotspot.all.peaks.bed.gz"
## [24] "peaks/consolidated/narrowPeak/E085-DNase.hotspot.all.peaks.bed.gz"
## [25] "peaks/consolidated/narrowPeak/E086-DNase.hotspot.all.peaks.bed.gz"
## [26] "peaks/consolidated/narrowPeak/E088-DNase.hotspot.all.peaks.bed.gz"
## [27] "peaks/consolidated/narrowPeak/E089-DNase.hotspot.all.peaks.bed.gz"
## [28] "peaks/consolidated/narrowPeak/E090-DNase.hotspot.all.peaks.bed.gz"
## [29] "peaks/consolidated/narrowPeak/E091-DNase.hotspot.all.peaks.bed.gz"
## [30] "peaks/consolidated/narrowPeak/E092-DNase.hotspot.all.peaks.bed.gz"
## [31] "peaks/consolidated/narrowPeak/E093-DNase.hotspot.all.peaks.bed.gz"
## [32] "peaks/consolidated/narrowPeak/E094-DNase.hotspot.all.peaks.bed.gz"
## [33] "peaks/consolidated/narrowPeak/E097-DNase.hotspot.all.peaks.bed.gz"
## [34] "peaks/consolidated/narrowPeak/E098-DNase.hotspot.all.peaks.bed.gz"
## [35] "peaks/consolidated/narrowPeak/E100-DNase.hotspot.all.peaks.bed.gz"
## [36] "peaks/consolidated/narrowPeak/E109-DNase.hotspot.all.peaks.bed.gz"
## [37] "peaks/consolidated/narrowPeak/E029-DNase.hotspot.all.peaks.bed.gz"
## [38] "peaks/consolidated/narrowPeak/E034-DNase.hotspot.all.peaks.bed.gz"
## [39] "peaks/consolidated/narrowPeak/E046-DNase.hotspot.all.peaks.bed.gz"


"https://bioconductorhubs.blob.core.windows.net/annotationhub/"
## 61 files
## ncbi and ensebml are Bioconductor maintained and added either at release or
## when new ensembl version is available
##
## EuPathDB and ontoProcData are contributed
##
## Could be an issue with special characters that needs manual adjustmenet with
## url coding 
## Or could have had corrupt upload/not existed and remove from database

ah_bad[which(ah_bad$location_prefix == "https://bioconductorhubs.blob.core.windows.net/annotationhub/"), "rdatapath"]
##  [1] "ncbi/org.Coprinopsis_cinerea_okayama7#130.eg.sqlite"                                                 
##  [2] "ncbi/org.Eremothecium_cymbalariae_DBVPG#7215.eg.sqlite"                                              
##  [3] "ncbi/uniprot/org.Coprinopsis_cinerea_okayama7#130.eg.sqlite"                                         
##  [4] "ncbi/uniprot/org.Eremothecium_cymbalariae_DBVPG#7215.eg.sqlite"                                      
##  [5] "ncbi/uniprot/3.5/org.Coprinopsis_cinerea_okayama7#130.eg.sqlite"                                     
##  [6] "ncbi/uniprot/3.5/org.Eremothecium_cymbalariae_DBVPG#7215.eg.sqlite"                                  
##  [7] "ncbi/uniprot/3.6/org.Coprinopsis_cinerea_okayama7#130.eg.sqlite"                                     
##  [8] "ncbi/uniprot/3.6/org.Eremothecium_cymbalariae_DBVPG#7215.eg.sqlite"                                  
##  [9] "ensembl/release-92/fasta/cebus_capucinus/cdna/Cebus_capucinus.Cebus_imitator-1.0.cdna.all.2bit"      
## [10] "ensembl/release-92/fasta/cebus_capucinus/dna/Cebus_capucinus.Cebus_imitator-1.0.dna_rm.toplevel.2bit"
## [11] "ensembl/release-92/fasta/cebus_capucinus/dna/Cebus_capucinus.Cebus_imitator-1.0.dna_sm.toplevel.2bit"
## [12] "ensembl/release-92/fasta/cebus_capucinus/ncrna/Cebus_capucinus.Cebus_imitator-1.0.ncrna.2bit"        
## [13] "ncbi/uniprot/3.7/org.Coprinopsis_cinerea_okayama7#130.eg.sqlite"                                     
## [14] "ncbi/uniprot/3.7/org.Coprinus_cinereus_okayama7#130.eg.sqlite"                                       
## [15] "ncbi/uniprot/3.7/org.Eremothecium_cymbalariae_DBVPG#7215.eg.sqlite"                                  
## [16] "EuPathDB/OrgDb/3.8/org.Coprinopsis_cinerea_okayama7#130.fungi.db.sqlite"                             
## [17] "EuPathDB/OrgDb/3.8/org.Albugo_laibachii_Nc14.fungi.db.sqlite"                                        
## [18] "EuPathDB/GRanges/3.8/GRanges.Coprinopsis_cinerea_okayama7#130.fungidb39.rda"                         
## [19] "ncbi/uniprot/3.8/org.Coprinopsis_cinerea_okayama7#130.eg.sqlite"                                     
## [20] "ncbi/uniprot/3.8/org.Coprinus_cinereus_okayama7#130.eg.sqlite"                                       
## [21] "ncbi/uniprot/3.9/org.Coprinopsis_cinerea_okayama7#130.eg.sqlite"                                     
## [22] "ncbi/uniprot/3.9/org.Coprinus_cinereus_okayama7#130.eg.sqlite"                                       
## [23] "ncbi/uniprot/3.10/org.Coprinopsis_cinerea_okayama7#130.eg.sqlite"                                    
## [24] "ncbi/uniprot/3.10/org.Coprinus_cinereus_okayama7#130.eg.sqlite"                                      
## [25] "ncbi/uniprot/3.11/org.Coprinopsis_cinerea_okayama7#130.eg.sqlite"                                    
## [26] "ncbi/uniprot/3.11/org.Coprinus_cinereus_okayama7#130.eg.sqlite"                                      
## [27] "ensembl/release-100/fasta/cercocebus_atys/cdna/Cercocebus_atys.Caty_1.0.cdna.all.2bit"               
## [28] "ensembl/release-100/fasta/equus_caballus/cdna/Equus_caballus.EquCab3.0.cdna.all.2bit"                
## [29] "ensembl/release-100/fasta/ficedula_albicollis/ncrna/Ficedula_albicollis.FicAlb_1.4.ncrna.2bit"       
## [30] "ensembl/release-100/fasta/fukomys_damarensis/ncrna/Fukomys_damarensis.DMR_v1.0.ncrna.2bit"           
## [31] "ensembl/release-100/fasta/hippocampus_comes/cdna/Hippocampus_comes.H_comes_QL1_v1.cdna.all.2bit"     
## [32] "ensembl/release-100/fasta/nothoprocta_perdicaria/cdna/Nothoprocta_perdicaria.notPer1.cdna.all.2bit"  
## [33] "ensembl/release-100/fasta/taeniopygia_guttata/cdna/Taeniopygia_guttata.bTaeGut1_v1.p.cdna.all.2bit"  
## [34] "ncbi/uniprot/3.12/org.Coprinopsis_cinerea_okayama7#130.eg.sqlite"                                    
## [35] "ncbi/uniprot/3.12/org.Coprinus_cinereus_okayama7#130.eg.sqlite"                                      
## [36] "ncbi/uniprot/3.13/org.Coprinopsis_cinerea_okayama7#130.eg.sqlite"                                    
## [37] "ncbi/uniprot/3.13/org.Coprinus_cinereus_okayama7#130.eg.sqlite"                                      
## [38] "ncbi/uniprot/3.14/org.Coprinopsis_cinerea_okayama7#130.eg.sqlite"                                    
## [39] "ncbi/uniprot/3.14/org.Coprinus_cinereus_okayama7#130.eg.sqlite"                                      
## [40] "ncbi/uniprot/3.15/org.Coprinopsis_cinerea_okayama7#130.eg.sqlite"                                    
## [41] "ncbi/uniprot/3.15/org.Coprinus_cinereus_okayama7#130.eg.sqlite"                                      
## [42] "ncbi/uniprot/3.16/org.Coprinopsis_cinerea_okayama7#130.eg.sqlite"                                    
## [43] "ncbi/uniprot/3.16/org.Coprinus_cinereus_okayama7#130.eg.sqlite"                                      
## [44] "ontoProcData/mondo_2022.12.01.rda"                                                                   
## [45] "ncbi/uniprot/3.17/org.Esox_lucius.eg.sqlite"                                                         
## [46] "ncbi/uniprot/3.17/org.Rhinolophus_sinicus.eg.sqlite"                                                 
## [47] "ncbi/uniprot/3.17/org.Hippoglossus_hippoglossus.eg.sqlite"                                           
## [48] "ncbi/uniprot/3.17/org.Agelaius_phoeniceus.eg.sqlite"                                                 
## [49] "ncbi/uniprot/3.17/org.Leptinotarsa_decemlineata.eg.sqlite"                                           
## [50] "ncbi/uniprot/3.17/org.Leptinotarsa_decimlineata.eg.sqlite"                                           
## [51] "ncbi/uniprot/3.17/org.Struthio_australis.eg.sqlite"                                                  
## [52] "ncbi/uniprot/3.17/org.Struthio_camelus_australis.eg.sqlite"                                          
## [53] "ncbi/uniprot/3.17/org.Drosophila_mauritiana.eg.sqlite"                                               
## [54] "ncbi/uniprot/3.17/org.Geospiza_fortis.eg.sqlite"                                                     
## [55] "ncbi/uniprot/3.17/org.Stilbospora_angustata.eg.sqlite"                                               
## [56] "ncbi/uniprot/3.17/org.Lophyrus_lecontei.eg.sqlite"                                                   
## [57] "ncbi/uniprot/3.17/org.Neodiprion_lecontei.eg.sqlite"                                                 
## [58] "ncbi/uniprot/3.17/org.Papilio_brassicae.eg.sqlite"                                                   
## [59] "ncbi/uniprot/3.17/org.Coprinopsis_cinerea_okayama7#130.eg.sqlite"                                    
## [60] "ncbi/uniprot/3.17/org.Coprinus_cinereus_okayama7#130.eg.sqlite"                                      



"http://www.pazar.info/"
## 91 files
## PazarImportPreparer, Bioconductor maintained
## url hangs
## recipe is blank
## remove?



"ftp://ftp.ensembl.org/pub/"
## 12231 files
## seems release-84 to release-109
## these are gtf? are these granges on the fly?
## false positive?

## ---- eval=FALSE--------------------------------------------------------------
#  .onLoad <- function(libname, pkgname) {
#  	fl <- system.file("extdata", "metadata.csv", package=pkgname)
#  	titles <- read.csv(fl, stringsAsFactors=FALSE)$Title
#  	createHubAccessors(pkgname, titles)
#  }

## ---- eval=FALSE--------------------------------------------------------------
#  library(ExperimentHub)
#  eh <- ExperimentHub()
#  myfiles <- query(eh, "PACKAGENAME")
#  myfiles[[1]]        ## load the first resource in the list
#  myfiles[["EH123"]]  ## load by EH id

## ---- eval=FALSE--------------------------------------------------------------
#  resourceA(metadata = FALSE) ## data are loaded
#  resourceA(metadata = TRUE)  ## metadata are displayed

---
title: "ExperimentHub: Access the ExperimentHub Web Service"
output:
  BiocStyle::html_document:
    toc: true
vignette: >
  % \VignetteIndexEntry{ExperimentHub: Access the ExperimentHub Web Service}
  % \VignetteDepends{ExperimentHub}
  % \VignetteEngine{knitr::rmarkdown}
  % \VignetteEncoding{UTF-8}
---

```{r style, echo = FALSE, results = 'asis'}
BiocStyle::markdown()
```

The `ExperimentHub` server provides easy _R / Bioconductor_ access to
large files of data.

# ExperimentHub objects

The `r Biocpkg("ExperimentHub")` package provides a client interface
to resources stored at the ExperimentHub web service. It has similar
functionality to `r Biocpkg("AnnotationHub")` package.

```{r library, message=FALSE}
library(ExperimentHub)
```

The `r Biocpkg("ExperimentHub")` package is straightforward to use.
Create an `ExperimentHub` object

```{r ExperimentHub}
eh = ExperimentHub()
```

Now at this point you have already done everything you need in order
to start retrieving experiment data.  For most operations, using the
`ExperimentHub` object should feel a lot like working with a familiar
`list` or `data.frame` and has all of the functionality of an `Hub`
object like  `r Biocpkg("AnnotationHub")` package's `AnnotationHub`
object.

Lets take a minute to look at the show method for the hub object eh

```{r show}
eh
```

You can see that it gives you an idea about the different types of data that are present inside the hub. You can see where the data is coming from (dataprovider), as well as what species have samples present (species), what kinds of R data objects could be returned (rdataclass). We can take a closer look at all the kinds of data providers that are available by simply looking at the contents of dataprovider as if it were the column of a data.frame object like this:

```{r dataprovider}
head(unique(eh$dataprovider))
```

In the same way, you can also see data from different species inside the hub by looking at the contents of species like this:

```{r species}
head(unique(eh$species))
```

And this will also work for any of the other types of metadata present.  You can learn which kinds of metadata are available by simply hitting the tab key after you type 'eh$'.  In this way you can explore for yourself what kinds of data are present in the hub right from the command line. This interface also allows you to access the hub programmatically to extract data that matches a particular set of criteria.

Another valuable types of metadata to pay attention to is the rdataclass.

```{r rdataclass}
head(unique(eh$rdataclass))
```

The rdataclass allows you to see which kinds of R objects the hub will return to you.  This kind of information is valuable both as a means to filter results and also as a means to explore and learn about some of the kinds of `ExperimentHub` objects that are widely available for the project.  Right now this is a pretty short list, but over time it should grow as we support more of the different kinds of `ExperimentHub` objects via the hub.

Now let's try getting the data files associated with the `r
Biocpkg("GSE103322")` package using the query method. The query method lets you
search rows for specific strings, returning an `ExperimentHub` instance with
just the rows matching the query. The `ExperimentHub::package()` function
indicates the package responsible for original preparation and documentation of
the ExperimentHub data, and is a useful resource to find out more about the
data. This package may also depend, import or suggest other packages required
for manipulating the data, and for this reason the package may need to be
installed and loaded to use the resource. The package is also available in the
(cryptically named) `preparerclass` column of the metadata of the ExperimentHub
object.

One can get chain files for Drosophila melanogaster from UCSC with:

```{r alpine}
gseData <- query(eh, "GSE103322")
gseData
```
Query has worked and you can now see that the only data present is provided by
the `r Biocpkg("GSE103322")` package.

The metadata underlying this hub object can be retrieved by you

```{r show2}
gseData$preparerclass
df <- mcols(gseData)
```

By default the show method will only display the first 5 and last 5  rows.
There are hundreds of records present in the hub.

```{r length}
length(eh)
```

Let's look at another example, where we pull down only data
from the hub for species "mus musculus".

```{r subset}
mm <- query(eh, "mus musculus")
mm
```

## Interactive retrieval of resources with BiocHubsShiny

We can perform the same query using the `BiocHubsShiny()` function from the
eponymous package. In the 'species' field we can enter `musculus` as the
search term.

```{r BiocHubsShiny, eval=FALSE}
BiocHubsShiny::BiocHubsShiny()
```

```{r,echo=FALSE,fig.cap="BiocHubsShiny query with species term 'musculus'",out.width="100%"}
knitr::include_graphics("../man/figures/mm10_BiocHubsShiny.png")
```

# Using `ExperimentHub` to retrieve data

Looking back at our GSE103322 file example, if we are interested in the first file, we can get its metadata using

```{r dm2}
gseData
gseData["EH5419"]
```

We can download the file using

```{r dm3}
gseData[["EH5419"]]
```

Each file is retrieved from the ExperimentHub server and the file is
also cached locally, so that the next time you need to retrieve it,
it should download much more quickly.

# Configuring `ExperimentHub` objects

When you create the `ExperimentHub` object, it will set up the object
for you with some default settings.  See `?ExperimentHub` for ways to
customize the hub source, the local cache, and other instance-specific
options, and `?getExperimentHubOption` to get or set package-global
options for use across sessions.

If you look at the object you will see some helpful information about
it such as where the data is cached and where online the hub server is
set to.

```{r show-2}
eh
```

By default the `ExperimentHub` object is set to the latest
`snapshotData` and a snapshot version that matches the version of
_Bioconductor_ that you are using. You can also learn about these data
with the appropriate methods.

```{r snapshot}
snapshotDate(eh)
```

If you are interested in using an older version of a snapshot, you can
list previous versions with the `possibleDates()` like this:

```{r possibleDates}
pd <- possibleDates(eh)
pd
```

Set the dates like this:

```{r setdate, eval=FALSE}
snapshotDate(ah) <- pd[1]
```



# Creating an ExperimentHub Package or Converting to an ExperimentHub Package

Please see `r Biocpkg("HubPub")` Vignette "CreateAHubPackage".
```
vignette("CreateAHubPackage", package="HubPub")
```

# Troubleshooting

Please see `r Biocpkg("AnnotationHub")` vignette "TroubleshootingTheHubs".
```
vignette("TroubleshootingTheHubs", package="AnnotationHub")
```

# Accessing Behind A Proxy

The ExperimentHub and AnnotationHub use CRAN package `httr2` functions for accessing and downloading
web resources. This can be problematic if operating behind a
proxy. You can pass proxy information for Hubs by setting the options like the following:

```{r eval=FALSE}
proxy <- "http://my_user:my_password@myproxy:8080"
AnnotationHub::setAnnotationHubOption("PROXY", proxy)
## or 
ExperimentHub::setExperimentHubOption("PROXY", proxy)
```

Unfortunately unlike the previously used `httr::set_config`, there is no
option to globally set the proxy for httr2 requests. To get around this you can
also set a system wide environment variable "ANNOTATION_HUB_PROXY" for
AnnotationHub, "EXPERIMENT_HUB_PROXY" for ExperimentHub, or "HUB_PROXY" that
will work across all Hub classes (e.g. AnnotationHub and ExperimentHub)}

# Other configuration options for resource downloading

As mentioned previously, There is no global option like `httr::set_config` to
set configuration options when using httr2. A `config` argument may be passed to
`[[` and `cache` functions. This argument is a R list object that will be passed
to `httr2::req_options`. The names of the items should be valid curl options as
defined in `curl::curl_options`.

```{r eval=FALSE}
ssl_opts <- list(verbose = 1L,ssl_verifypeer = 0L, ssl_verifyhost = 0L)

```

# Group Hub/Cache Access

The situation may occur where a hub is desired to be shared across multiple
users on a system.  This presents permissions errors.  To allow access to
multiple users create a group that the users belong to and that the underlying
hub cache belongs too. Permissions of potentially two files need to be altered depending
on what you would like individuals to be able to accomplish with the hub. A
read-only hub will require manual manipulatios of the file
BiocFileCache.sqlite.LOCK so that the group permissions are `g+rw`. To allow
users to download files to the shared hub, both the
BiocFileCache.sqlite.LOCK file and the BiocFileCache.sqlite file will need group
permissions to `g+rw`. Please google how to create a user group for your system
of interest. To find the location of the hub cache to be able to change the group
and file permissions, you may run the following in R `eh = ExperimentHub();
hubCache(eh)`. For quick reference in linux you will use `chown
currentuser:newgroup` to change the group and `chmod` to change the file
permissions: `chmod 660` or `chmod g+rw` should accomplish the correct
permissions.



# Default Caching Location Update


As of ExperimentHub version > 1.17.2, the default caching location has
changed. The default cache is now controlled by the function `tools::R_user_dir`
instead of `rappdirs::user_cache_dir`.  Users who have utilized the default
ExperimentHub location, to continue using the created cache, must move the cache and its
files to the new default location,  create a system environment variable to
point to the old location, or delete and start a new cache.

## Option 1: Move cache and files to new location

The following steps can be used to move the files to the new location:

1. Determine the old location by running the following in R
   `rappdirs::user_cache_dir(appname="ExperimentHub")`

2. Determine the new location by running the following in R
   `tools::R_user_dir("ExperimentHub", which="cache")`

3. Move the files to the new location. You can do this manually or do the
following steps in R. Remember if you have a lot of cached files, this may take
awhile.

```{r, eval=FALSE}

       # make sure you have permissions on the cache/files
       # use at own risk


       moveFiles<-function(package){
	   olddir <- path.expand(rappdirs::user_cache_dir(appname=package))
	   newdir <- tools::R_user_dir(package, which="cache")
	   dir.create(path=newdir, recursive=TRUE)
	   files <- list.files(olddir, full.names =TRUE)
	   moveres <- vapply(files,
	       FUN=function(fl){
		   filename = basename(fl)
		   newname = file.path(newdir, filename)
		   file.rename(fl, newname)
	       },
	       FUN.VALUE = logical(1))
	   if(all(moveres)) unlink(olddir, recursive=TRUE)
       }


       package="ExperimentHub"
       moveFiles(package)

```

## Option 2:  Create a system environment variable

A user may set the system environment variable `EXPERIMENT_HUB_CACHE` to control
the default location of the cache. Setting system environment variables can vary
depending on the operating system; we suggest using google to find appropriate
instructions per your operating system.

You will want to set the variable to the results of running the following in R:

```{r, eval=FALSE}
path.expand(rappdirs::user_cache_dir(appname="ExperimentHub"))
```

**NOTE:** R has `Sys.setenv` however that will only set the variable for that R
  session. It would not be available or recognized in future R sessions. It is
  important to set the variable as a global user-wide or system-wide
  environment variable so it persists in all future logins to your system.

## Option 3: Delete the old cache

Lastly, if a user does not care about the already existing default cache, the
old location may be deleted to move forward with the new default location. This
option should be used with caution. Once deleted, old cached resources will no
longer be available and have to be re-downloaded.

One can do this manually by navigating to the location indicated in the ERROR
message as `Problematic cache:` and deleting the folder and all its content.


The following can be done to delete through R code:

**CAUTION** This will remove the old cache and all downloaded resources. All
  resources will have to be re-downloaded after executing this code.

```{r, eval=FALSE}
library(ExperimentHub)
oldcache = path.expand(rappdirs::user_cache_dir(appname="ExperimentHub"))
setExperimentHubOption("CACHE", oldcache)
eh = ExperimentHub(localHub=TRUE)

## removes old location and all resources
removeCache(eh, ask=FALSE)

## create the new default caching location
newcache = tools::R_user_dir("ExperimentHub", which="cache")
setExperimentHubOption("CACHE", newcache)
eh = ExperimentHub()
```

# Session info

```{r sessionInfo}
sessionInfo()
```

---
title: "ExperimentHub: Access the ExperimentHub Web Service"
output:
  BiocStyle::html_document:
    toc: true
vignette: >
  % \VignetteIndexEntry{ExperimentHub: Access the ExperimentHub Web Service}
  % \VignetteDepends{ExperimentHub}
  % \VignetteEngine{knitr::rmarkdown}
  % \VignetteEncoding{UTF-8}
---

```{r style, echo = FALSE, results = 'asis'}
BiocStyle::markdown()
```

The `ExperimentHub` server provides easy _R / Bioconductor_ access to
large files of data.

# ExperimentHub objects

The `r Biocpkg("ExperimentHub")` package provides a client interface
to resources stored at the ExperimentHub web service. It has similar
functionality to `r Biocpkg("AnnotationHub")` package.

```{r library, message=FALSE}
library(ExperimentHub)
```

The `r Biocpkg("ExperimentHub")` package is straightforward to use.
Create an `ExperimentHub` object

```{r ExperimentHub}
eh = ExperimentHub()
```

Now at this point you have already done everything you need in order
to start retrieving experiment data.  For most operations, using the
`ExperimentHub` object should feel a lot like working with a familiar
`list` or `data.frame` and has all of the functionality of an `Hub`
object like  `r Biocpkg("AnnotationHub")` package's `AnnotationHub`
object.

Lets take a minute to look at the show method for the hub object eh

```{r show}
eh
```

You can see that it gives you an idea about the different types of data that are present inside the hub. You can see where the data is coming from (dataprovider), as well as what species have samples present (species), what kinds of R data objects could be returned (rdataclass). We can take a closer look at all the kinds of data providers that are available by simply looking at the contents of dataprovider as if it were the column of a data.frame object like this:

```{r dataprovider}
head(unique(eh$dataprovider))
```

In the same way, you can also see data from different species inside the hub by looking at the contents of species like this:

```{r species}
head(unique(eh$species))
```

And this will also work for any of the other types of metadata present.  You can learn which kinds of metadata are available by simply hitting the tab key after you type 'eh$'.  In this way you can explore for yourself what kinds of data are present in the hub right from the command line. This interface also allows you to access the hub programmatically to extract data that matches a particular set of criteria.

Another valuable types of metadata to pay attention to is the rdataclass.

```{r rdataclass}
head(unique(eh$rdataclass))
```

The rdataclass allows you to see which kinds of R objects the hub will return to you.  This kind of information is valuable both as a means to filter results and also as a means to explore and learn about some of the kinds of `ExperimentHub` objects that are widely available for the project.  Right now this is a pretty short list, but over time it should grow as we support more of the different kinds of `ExperimentHub` objects via the hub.

Now let's try getting the data files associated with the `r
Biocpkg("alpineData")` package using the query method. The query method lets you
search rows for specific strings, returning an `ExperimentHub` instance with
just the rows matching the query. The `ExperimentHub::package()` function
indicates the package responsible for original preparation and documentation of
the ExperimentHub data, and is a useful resource to find out more about the
data. This package may also depend, import or suggest other packages required
for manipulating the data, and for this reason the package may need to be
installed and loaded to use the resource. The package is also available in the
(cryptically named) `preparerclass` column of the metadata of the ExperimentHub
object.

One can get chain files for Drosophila melanogaster from UCSC with:

```{r alpine}
apData <- query(eh, "alpineData")
apData
```
Query has worked and you can now see that the only data present is provided by
the `r Biocpkg("alpineData")` package.

The metadata underlying this hub object can be retrieved by you

```{r show2}
apData$preparerclass
df <- mcols(apData)
```

By default the show method will only display the first 5 and last 5  rows.
There are hundreds of records present in the hub.

```{r length}
length(eh)
```

Let's look at another example, where we pull down only data
from the hub for species "mus musculus".

```{r subset}
mm <- query(eh, "mus musculus")
mm
```

## Interactive retrieval of resources with BiocHubsShiny

We can perform the same query using the `BiocHubsShiny()` function from the
eponymous package. In the 'species' field we can enter `musculus` as the
search term.

```{r BiocHubsShiny, eval=FALSE}
BiocHubsShiny::BiocHubsShiny()
```

```{r,echo=FALSE,fig.cap="BiocHubsShiny query with species term 'musculus'",out.width="100%"}
knitr::include_graphics("../man/figures/mm10_BiocHubsShiny.png")
```

# Using `ExperimentHub` to retrieve data

Looking back at our alpineData file example, if we are interested in the first file, we can get its metadata using

```{r dm2}
apData
apData["EH166"]
```

We can download the file using

```{r dm3}
apData[["EH166"]]
```

Each file is retrieved from the ExperimentHub server and the file is
also cached locally, so that the next time you need to retrieve it,
it should download much more quickly.

# Configuring `ExperimentHub` objects

When you create the `ExperimentHub` object, it will set up the object
for you with some default settings.  See `?ExperimentHub` for ways to
customize the hub source, the local cache, and other instance-specific
options, and `?getExperimentHubOption` to get or set package-global
options for use across sessions.

If you look at the object you will see some helpful information about
it such as where the data is cached and where online the hub server is
set to.

```{r show-2}
eh
```

By default the `ExperimentHub` object is set to the latest
`snapshotData` and a snapshot version that matches the version of
_Bioconductor_ that you are using. You can also learn about these data
with the appropriate methods.

```{r snapshot}
snapshotDate(eh)
```

If you are interested in using an older version of a snapshot, you can
list previous versions with the `possibleDates()` like this:

```{r possibleDates}
pd <- possibleDates(eh)
pd
```

Set the dates like this:

```{r setdate, eval=FALSE}
snapshotDate(ah) <- pd[1]
```



# Creating an ExperimentHub Package or Converting to an ExperimentHub Package

Please see `r Biocpkg("HubPub")` Vignette "CreateAHubPackage".
```
vignette("CreateAHubPackage", package="HubPub")
```

# Troubleshooting

Please see `r Biocpkg("AnnotationHub")` vignette "TroubleshootingTheHubs".
```
vignette("TroubleshootingTheHubs", package="AnnotationHub")
```

# Accessing Behind A Proxy

The ExperimentHub and AnnotationHub use CRAN package `httr2` functions for accessing and downloading
web resources. This can be problematic if operating behind a
proxy. You can pass proxy information for Hubs by setting the options like the following:

```{r eval=FALSE}
proxy <- "http://my_user:my_password@myproxy:8080"
AnnotationHub::setAnnotationHubOption("PROXY", proxy)
## or 
ExperimentHub::setExperimentHubOption("PROXY", proxy)
```

Unfortunately unlike the previously used `httr::set_config`, there is no
option to globally set the proxy for httr2 requests. To get around this you can
also set a system wide environment variable "ANNOTATION_HUB_PROXY" for
AnnotationHub, "EXPERIMENT_HUB_PROXY" for ExperimentHub, or "HUB_PROXY" that
will work across all Hub classes (e.g. AnnotationHub and ExperimentHub)}

# Other configuration options for resource downloading

As mentioned previously, There is no global option like `httr::set_config` to
set configuration options when using httr2. A `config` argument may be passed to
`[[` and `cache` functions. This argument is a R list object that will be passed
to `httr2::req_options`. The names of the items should be valid curl options as
defined in `curl::curl_options`.

```{r eval=FALSE}
ssl_opts <- list(verbose = 1L,ssl_verifypeer = 0L, ssl_verifyhost = 0L)

```

# Group Hub/Cache Access

The situation may occur where a hub is desired to be shared across multiple
users on a system.  This presents permissions errors.  To allow access to
multiple users create a group that the users belong to and that the underlying
hub cache belongs too. Permissions of potentially two files need to be altered depending
on what you would like individuals to be able to accomplish with the hub. A
read-only hub will require manual manipulatios of the file
BiocFileCache.sqlite.LOCK so that the group permissions are `g+rw`. To allow
users to download files to the shared hub, both the
BiocFileCache.sqlite.LOCK file and the BiocFileCache.sqlite file will need group
permissions to `g+rw`. Please google how to create a user group for your system
of interest. To find the location of the hub cache to be able to change the group
and file permissions, you may run the following in R `eh = ExperimentHub();
hubCache(eh)`. For quick reference in linux you will use `chown
currentuser:newgroup` to change the group and `chmod` to change the file
permissions: `chmod 660` or `chmod g+rw` should accomplish the correct
permissions.



# Default Caching Location Update


As of ExperimentHub version > 1.17.2, the default caching location has
changed. The default cache is now controlled by the function `tools::R_user_dir`
instead of `rappdirs::user_cache_dir`.  Users who have utilized the default
ExperimentHub location, to continue using the created cache, must move the cache and its
files to the new default location,  create a system environment variable to
point to the old location, or delete and start a new cache.

## Option 1: Move cache and files to new location

The following steps can be used to move the files to the new location:

1. Determine the old location by running the following in R
   `rappdirs::user_cache_dir(appname="ExperimentHub")`

2. Determine the new location by running the following in R
   `tools::R_user_dir("ExperimentHub", which="cache")`

3. Move the files to the new location. You can do this manually or do the
following steps in R. Remember if you have a lot of cached files, this may take
awhile.

```{r, eval=FALSE}

       # make sure you have permissions on the cache/files
       # use at own risk


       moveFiles<-function(package){
	   olddir <- path.expand(rappdirs::user_cache_dir(appname=package))
	   newdir <- tools::R_user_dir(package, which="cache")
	   dir.create(path=newdir, recursive=TRUE)
	   files <- list.files(olddir, full.names =TRUE)
	   moveres <- vapply(files,
	       FUN=function(fl){
		   filename = basename(fl)
		   newname = file.path(newdir, filename)
		   file.rename(fl, newname)
	       },
	       FUN.VALUE = logical(1))
	   if(all(moveres)) unlink(olddir, recursive=TRUE)
       }


       package="ExperimentHub"
       moveFiles(package)

```

## Option 2:  Create a system environment variable

A user may set the system environment variable `EXPERIMENT_HUB_CACHE` to control
the default location of the cache. Setting system environment variables can vary
depending on the operating system; we suggest using google to find appropriate
instructions per your operating system.

You will want to set the variable to the results of running the following in R:

```{r, eval=FALSE}
path.expand(rappdirs::user_cache_dir(appname="ExperimentHub"))
```

**NOTE:** R has `Sys.setenv` however that will only set the variable for that R
  session. It would not be available or recognized in future R sessions. It is
  important to set the variable as a global user-wide or system-wide
  environment variable so it persists in all future logins to your system.

## Option 3: Delete the old cache

Lastly, if a user does not care about the already existing default cache, the
old location may be deleted to move forward with the new default location. This
option should be used with caution. Once deleted, old cached resources will no
longer be available and have to be re-downloaded.

One can do this manually by navigating to the location indicated in the ERROR
message as `Problematic cache:` and deleting the folder and all its content.


The following can be done to delete through R code:

**CAUTION** This will remove the old cache and all downloaded resources. All
  resources will have to be re-downloaded after executing this code.

```{r, eval=FALSE}
library(ExperimentHub)
oldcache = path.expand(rappdirs::user_cache_dir(appname="ExperimentHub"))
setExperimentHubOption("CACHE", oldcache)
eh = ExperimentHub(localHub=TRUE)

## removes old location and all resources
removeCache(eh, ask=FALSE)

## create the new default caching location
newcache = tools::R_user_dir("ExperimentHub", which="cache")
setExperimentHubOption("CACHE", newcache)
eh = ExperimentHub()
```

# Session info

```{r sessionInfo}
sessionInfo()
```

---
title: "Creating An ExperimentHub Package"
author: "Valerie Obenchain and Lori Shepherd"
date: "Modified: Aug 2019. Compiled: `r format(Sys.Date(), '%d %b %Y')`"
output:
  BiocStyle::html_document:
    toc: true
    toc_depth: 2
vignette: >
  %\VignetteIndexEntry{Creating An ExperimentHub Package}
  %\VignetteEngine{knitr::rmarkdown}
---

# Overview

`ExperimentHubData` provides tools to add or modify resources in
Bioconductor's `ExperimentHub`. This 'hub' houses curated data from courses,
publications or experiments.  The resources are generally not files of raw data
(as can be the case in `AnnotationHub`) but instead are `R` / `Bioconductor`
objects such as GRanges, SummarizedExperiment, data.frame etc.  Each resource
has associated metadata that can be searched through the `ExperimentHub` client interface.

# New resources

Resources are contributed to `ExperimentHub` in the form of a package.  The
package contains the resource metadata, man pages, vignette and any supporting
`R` functions the author wants to provide.  This is a similar design to the
existing `Bioconductor` experimental data packages except the data are
stored in AWS S3 buckets or publicly accessibly sites instead of the data/
directory of the package.

Below are the steps required for adding new resources.

## Notify `Bioconductor` team member

The man page and vignette examples in the data experiment package will not work until
the data are available in `ExperimentHub`. Adding the data to AWS S3 and the
metadata to the production database involves assistance from a `Bioconductor`
team member. The metadata.csv file will have to be created before the data can
officially be added to the hub (See inst/extdata section below). Please read the
section of "Storage of Data Files".

## Building the data experiment package

When a resource is downloaded from `ExperimentHub` the associated data experiment
package is loaded in the workspace making the man pages and vignettes readily
available. Because documentation plays an important role in understanding these
curated resources please take the time to develop clear man pages and a
detailed vignette. These documents provide essential background to the user and
guide appropriate use the of resources.

Below is an outline of package organization. The files listed are required
unless otherwise stated.

### `inst/extdata/`

- `metadata.csv`:
  This file contains the metadata in the format of one row per resource
  to be added to the `ExperimentHub` database. The file should be generated
  from the code in inst/scripts/make-metadata.R where the final data are
  written out with `write.csv(..., row.names=FALSE)`. The required column
  names and data types are specified in
  `ExperimentHubData::makeExperimentHubMetadata` See
  ?`ExperimentHubData::makeExperimentHubMetadata` for details.

  An example data experiment package metadata.csv file can be found [here](https://github.com/Bioconductor/GSE62944/blob/master/inst/extdata/metadata.csv)

### `inst/scripts/`

- `make-data.R`:
  A script describing the steps involved in making the data object(s). This
  includes where the original data were downloaded from, pre-processing,
  and how the final R object was made. Include a description of any
  steps performed outside of `R` with third party software. It is encouraged to
  serialize Data objects with `save()` with the .rda extension on the filename
  but not strictly necessary. If the data is provided in another format an
  appropriate loading method may need to be implemented. Please advise when
  reaching out for "Uploading Data to S3".

- `make-metadata.R`:
  A script to make the metadata.csv file located in inst/extdata of the
  package. See ?`ExperimentHubData::makeExperimentHubMetadata` for a description
  of expected fields and data types.
  `ExperimentHubData::makeExperimentHubMetadata()` can be used to validate the
  metadata.csv file before submitting the package.

### `vignettes/`

- One or more vignettes describing analysis workflows.

### `R/`

- `zzz.R`: Optional. You can include a `.onLoad()` function in a zzz.R file that
  exports each resource name (i.e., title) into a function. This allows the data
  to be loaded by name, e.g., `resouce123()`.

    ```{r, eval=FALSE}
    .onLoad <- function(libname, pkgname) {
	fl <- system.file("extdata", "metadata.csv", package=pkgname)
	titles <- read.csv(fl, stringsAsFactors=FALSE)$Title
	createHubAccessors(pkgname, titles)
    }
    ```

    `ExperimentHub::createHubAccessors()` and
    `ExperimentHub:::.hubAccessorFactory()` provide internal
    detail. The resource-named function has a single 'metadata'
    argument. When metadata=TRUE, the metadata are loaded (equivalent
    to single-bracket method on an ExperimentHub object) and when
    FALSE the full resource is loaded (equivalent to double-bracket
    method).

- `R/*.R`: Optional. Functions to enhance data exploration.

### `man/`

- package man page:
  The package man page serves as a landing point and should briefly describe
  all resources associated with the package. There should be an \alias
  entry for each resource title either on the package man page or individual
  man pages.

- resource man pages:
  Resources can be documented on the same page, grouped by common type
  or have their own dedicated man pages.

- document how data are loaded:
  Data can be accessed via the standard ExperimentHub interface with
  single and double-bracket methods, e.g.,

    ```{r, eval=FALSE}
    library(ExperimentHub)
    eh <- ExperimentHub()
    myfiles <- query(eh, "PACKAGENAME")
    myfiles[[1]]        ## load the first resource in the list
    myfiles[["EH123"]]  ## load by EH id
    ```

- If a `.onLoad()` function is used to export each resource as a function
  also document that method of loading, e.g.,

    ```{r, eval=FALSE}
    resourceA(metadata = FALSE) ## data are loaded
    resourceA(metadata = TRUE)  ## metadata are displayed
    ```

### `DESCRIPTION` / `NAMESPACE`

- The package should depend on and fully import ExperimentHub. If using the
  suggested `.onLoad()` function, import the utils package in the DESCRIPTION
  file and selectively importFrom(utils, read.csv) in the NAMESPACE.

- Package authors are encouraged to use the `ExperimentHub::listResources()` and
  `ExperimentHub::loadResource()` functions in their man pages and vignette.
  These helpers are designed to facilitate data discovery within a specific
  package vs within all of ExperimentHub.

- The biocViews should contain terms from [ExperimentData](http://bioconductor.org/packages/release/BiocViews.html#___ExperimentData)
and should also contain the term `ExperimentHub`.


## Data objects

Data are not formally part of the software package and are stored
separately in a publicly accessible hosted site or by Bioconductor in an AWS S3
buckets. The author should read the following section on "Storage of Data Files".


## Metadata

When you are satisfied with the representation of your resources in
make-metadata.R (which produces metadata.csv) the `Bioconductor` team
member will add the metadata to the production database. Confirm the data in
inst/exdata/metadata.csv are valid by running
ExperimentHubData:::makeExperimentHubMetadata() on your package. Please address
and warnings or errors.


## Package review

Once the data are in AWS S3 or public site and the metadata have been added to the
production database the man pages and vignette can be finalized. When the
package passes R CMD build and check it can be submitted to the
[package tracker](https://github.com/Bioconductor/Contributions) for
review. The package should be submitted without any of the data that is now
located remotely; This keeps the package light weight and minimual size while still
providing access to key large data files now stored remotely If the data files
were added to the github repository please see [removing large data files and
clean git
tree](http://bioconductor.org/developers/how-to/git/remove-large-data/) to
remove the large files and reduce package size.

Many times these data package are created as a suppliment to a software
package. There is a process for submitting [mulitple package under the same
issue](https://github.com/Bioconductor/Contributions#submitting-related-packages).



# Add additional resources

Metadata for new versions of the data can be added to the same package as they
become available.

* The titles for the new versions must be unique and not match the title of
  any resource currently in ExperimentHub. Good practice would be to
  include the version and / or genome build in the title.

*  Make data available: either on publicly accessible site or see section on
  "Uploading Data to S3"

* Update make-metadata.R with the new metadata information

* Generate a new metadata.csv file. The package should contain
  metadata for all versions of the data in ExperimentHub so the old file should
  remain.  When adding a new version it might be helpful to write a new csv file
  named by version, e.g., metadata_v84.csv, metadata_85.csv etc.

* Bump package version and commit to git

* Notify hubs@bioconductor.org that an update is ready and
  a team member will add the new metadata to the production database;
  new resources will not be visible in ExperimentHub until
  the metadata are added to the database.


Contact  hubs@bioconductor.org or maintainer@bioconductor.org with any
questions.

# Bug fixes

A bug fix may involve a change to the metadata, data resource or both.

## Update the resource

* The replacement resource must have the same name as the original and be at the
  same location (path)

* Notify hubs@bioconductor.org that you want to replace the data
  and make the files available: see section "Uploading Data to S3".

## Update the metadata

New metadata records can be added for new resources but modifying existing
records is discouraged. Record modification will only be done in the case of
bug fixes.

* Notify hubs@bioconductor.org that you want to change the metadata

* Update make-metadata.R with modified information and regenerate the
  metadata.csv file if necessary

* Bump the package version and commit to git

# Remove resources

Removing resources should be done with caution. The intent is that
ExperimentHub be a 'reproducible' resource by providing a stable snapshot
of the data. Data made available in Bioconductor version x.y.z should be
available for all versions greater than x.y.z. Unfortunately this is not
always possible. If you find it necessary to remove data from ExperimentHub
please contact hubs@bioconductor.org or maintainer@bioconductor.org for
assistance.

When a resource is removed from ExperimentHub the 'status' field in the
metadata is modified to explain why they are no longer available. Once
this status is changed the `ExperimentHub()` constructor will not list the
resource among the available ids. An attempt to extract the resource with
'[[' and the EH id will return an error along with the status message. The
function `getInfoOnIds` will display metadata information for any resource
including resources still in the database but no longer available.

# Storage of Data Files

The data should not be included in the package. This keeps the package light
weight and quick for a user to install. This allows the user to investigate functions and
documentation without downloading large data files and only proceeding with the
download when necessary. There are two options for storing data: Bioconductor
AWS S3 buckets or hosting the data elsewhere on a publicly accessible site. See
information below and choose the options that fits best for your situation.

## Hosting Data on a Publicly Accessible Site

Data can be accessed through the hubs from any publicly accessible site. The
metadata.csv file[s] created will need the column `Location_Prefix` to indicate
the hosted site. See more in the description of the metadata columns/fields
below but as a quick example if the link to the data file is
`ftp://mylocalserver/singlecellExperiments/dataSet1.Rds` an example breakdown of
the `Location_Prefix` and `RDataPath` for this entry in the metadata.csv file
would be `ftp://mylocalserver/` for the `Location_Prefix` and
`singlecellExperiments/dataSet1.Rds` for the `RDataPath`.

## Uploading Data to S3

Instead of providing the data files via dropbox, ftp, etc. we will grant
temporary access to an S3 bucket where you can upload your data. Please
email hubs@bioconductor.org for access.

You will be given access to the 'AnnotationContributor' user. Ensure that the
`AWS CLI` is installed on your machine. See instructions for installing `AWS
CLI` [here](https://aws.amazon.com/cli/). Once you  have requested access you
will be emailed a set of keys. There are two options to set the profile up for
AnnotationContributor

1.  Update your `.aws/config` file to include the following updating the keys
accordingly:

```
[profile AnnotationContributor]
output = text
region = us-east-1
aws_access_key_id = ****
aws_secret_access_key = ****
```
2. If you can't find the `.aws/config` file,  Run the following command entering
appropriate information from above

```
aws configure --profile AnnotationContributor
```

After the configuration is set you should be able to upload resources using

```
# To upload a full directory use recursive:

aws --profile AnnotationContributor s3 cp test_dir s3://annotation-contributor/test_dir --recursive --acl public-read

# To upload a single file

aws --profile AnnotationContributor s3 cp test_file.txt s3://annotation-contributor/test_file.txt --acl public-read

```

Please upload the data with the appropriate directory structure, including
subdirectories as necessary (i.e. top directory must be software package name,
then if applicable, subdirectories of versions, ...). Please also do not forget
to use the flag `--acl public-read`; This allows read access to the data file.

Once the upload is complete, email hubs@bioconductor.org to continue the
process. To add the data officially the data will need to be uploaded and the
metadata.csv file will need to be created in the github repository.


# Validating

The best way to validate record metadata is to  read inst/extdata/metadata.csv
with `ExperimentHubData::makeExperimentHubMetadata()`. If that is successful the
metadata are ready to go.

# Example metadata.csv file and more information

As described above the metadata.csv file (or multiple metadata.csv files) will
need to be created before the data can be added to the database. To ensure
proper formatting one should run `AnnotationHubData::makeAnnotationHubMetadata`
on the package with any/all metadata files, and address any ERRORs that
occur. Each object uploaded to S3 should have an entry in the metadata
file. Briefly, a description of the metadata columns required:

* Title: 'character(1)'. Name of the resource. This can be the exact file name
  (if self-describing) or a more complete description.
* Description: 'character(1)'. Brief description of the resource, similar to the
  'Description' field in a package DESCRIPTION file.
* BiocVersion: 'character(1)'. The first Bioconductor version the resource was
  made available for. Unless removed from the hub, the resource will be
  available for all versions greater than or equal to this field. This generally
  is the current devel version of Bioconductor.
* Genome: 'character(1)'. Genome. Can be NA.
* SourceType: 'character(1)'. Format of original data, e.g., FASTA, BAM, BigWig,
  etc. 'AnnotationHubData::getValidSourceTypes()' list currently acceptable values. If nothing seems
  appropiate for your data reach out to hubs@bioconductor.org
* SourceUrl: 'character(1)'. Location of original data files. Multiple urls
  should be provided as a comma separated string. If the data is simulated we
  recommend putting either a lab url or the url of the Bioconductor package.
* SourceVersion: 'character(1)'. Version of original data.
* Species: 'character(1)'. Species. For help on valid species see
  'getSpeciesList, validSpecies, or suggestSpecies.' Can be NA.
* TaxonomyId: 'character(1)'. Taxonomy ID. There are checks for valid taxonomyId
  given the Species which produce warnings. See GenomeInfoDb::loadTaxonomyDb()
  for full validation table. Can be NA.
* Coordinate_1_based: 'logical'. TRUE if data are 1-based. Can be NA.
* DataProvider: 'character(1)'. Name of company or institution that supplied the
  original (raw) data.
* Maintainer: 'character(1)'. Maintainer name and email in the following format:
  Maintainer Name <username@address>.
* RDataClass: 'character(1)'. R / Bioconductor class the data are stored in,
  e.g., GRanges, SummarizedExperiment, ExpressionSet etc.
* DispatchClass: 'character(1)'. Determines how data are loaded into R. The
  value for this field should be 'Rda' if the data were serialized with 'save()'
  and 'Rds' if serialized with 'saveRDS'. The filename should have the
  appropriate 'rda' or 'rds' extension. The function
  'AnnotationHub::DispatchClassList()' will output a matrix of currently
  implemented DispatchClass and brief description of utility. If a predefine
  class does not seem appropriate contact hubs@bioconductor.org
* Location_Prefix: 'character(1)'. Do not include this field if data are stored
  in the Bioconductor AWS S3; it will be generated automatically. If data will
  be accessed from a location other than AWS S3 this field should be the base
  url.
* RDataPath: 'character(1)'.This field should be the remainder of the path to
  the resource. The 'Location_Prefix' will be prepended to 'RDataPath' for the
  full path to the resource.  If the resource is stored in Bioconductor's AWS S3
  buckets, it should start with the name of the package associated with the
  metadata and should not start with a leading slash. It should include the
  resource file name.
* Tags: 'character() vector'.  'Tags' are search terms used to define a subset
  of resources in a 'Hub' object, e.g, in a call to 'query'.

Any additional columns in the metadata.csv file will be ignored but could be
included for internal reference.

More on Location_Prefix and RDataPath. These two fields make up the complete
file path url for downloading the data file. If using the Bioconductor AWS S3 bucket the
Location_Prefix should not be included in the metadata file[s] as this field
will be populated automatically.  The RDataPath will be the directory structure
you uploaded to S3. If you uploaded a directory `MyAnnotation/`, and that
directory had a subdirectory `v1/` that contained two files `counts.rds` and
`coldata.rds`, your metadata file will contain two rows and the RDataPaths would
be `MyAnnotation/v1/counts.rds` and `MyAnnotation/v1/coldata.rds`.  If you
host your data on a publicly accessible site you must include a base url as the
`Location_Prefix`. If your data file was at
`ftp://myinstiututeserver/biostats/project2/counts.rds`, your metadata file will
have one row and the `Location_Prefix` would be `ftp://myinstiututeserver/` and
the `RDataPath` would be `biostats/project2/counts.rds`.

This is a dummy example but hopefully it will give you an idea of the format.
Let's say I have a package myExperimentPackage and I upload two files one a
SummarizedExperiments of expression data saved as a .rda and the other a sqlite
database both considered simulated data.
You would want the following saved as a csv (comma seperated output)
but for easier view we show in a table:


Title | Description | BiocVersion | Genome | SourceType | SourceUrl | SourceVersion | Species | TaxonomyId | Coordinate_1_based | DataProvider | Maintainer | RDataClass | DispatchClass | RDataPath
---------------|----------------------------------------------|-----|-------------|-----|----------------------------------------------------------------------------------------------------|------------|-------------|------|------|---------|-------------------------------------------------------|-----------|----------|--------------------------------------------------------------------------
Simulated Expression Data  | Simulated Expression values for 12 samples and 12000 probles | 3.9 | NA   | Simulated | http://mylabshomepage | v1 | NA | NA | NA | http://bioconductor.org/packages/myExperimentPackage | Bioconductor Maintainer <maintainer@bioconductor.org> | SummarizedExperiment | Rda | myExperimentPackage/SEobject.rda
Simulated Database | Simulated Database containing gene mappings | 3.9 | hg19 | Simulated | http://bioconductor.org/packages/myExperimentPackage | v2 | Home sapiens | 9606 | NA | http://bioconductor.org/packages/myExperimentPackage | Bioconductor Maintainer <maintainer@bioconductor.org> | SQLiteConnection | SQLiteFile | myExperimentPackage/mydatabase.sqlite

---
title: "ExperimentHub: Access the ExperimentHub Web Service"
output:
  BiocStyle::html_document:
    toc: true
vignette: >
  % \VignetteIndexEntry{ExperimentHub: Access the ExperimentHub Web Service}
  % \VignetteDepends{ExperimentHub}
  % \VignetteEngine{knitr::rmarkdown}
  % \VignetteEncoding{UTF-8}
---

```{r style, echo = FALSE, results = 'asis'}
BiocStyle::markdown()
```

The `ExperimentHub` server provides easy _R / Bioconductor_ access to
large files of data.

# ExperimentHub objects

The `r Biocpkg("ExperimentHub")` package provides a client interface
to resources stored at the ExperimentHub web service. It has similar
functionality to `r Biocpkg("AnnotationHub")` package.

```{r library, message=FALSE}
library(ExperimentHub)
```

The `r Biocpkg("ExperimentHub")` package is straightforward to use.
Create an `ExperimentHub` object

```{r ExperimentHub}
eh = ExperimentHub()
```

Now at this point you have already done everything you need in order
to start retrieving experiment data.  For most operations, using the
`ExperimentHub` object should feel a lot like working with a familiar
`list` or `data.frame` and has all of the functionality of an `Hub`
object like  `r Biocpkg("AnnotationHub")` package's `AnnotationHub`
object.

Lets take a minute to look at the show method for the hub object eh

```{r show}
eh
```

You can see that it gives you an idea about the different types of data that are present inside the hub. You can see where the data is coming from (dataprovider), as well as what species have samples present (species), what kinds of R data objects could be returned (rdataclass). We can take a closer look at all the kinds of data providers that are available by simply looking at the contents of dataprovider as if it were the column of a data.frame object like this:

```{r dataprovider}
head(unique(eh$dataprovider))
```

In the same way, you can also see data from different species inside the hub by looking at the contents of species like this:

```{r species}
head(unique(eh$species))
```

And this will also work for any of the other types of metadata present.  You can learn which kinds of metadata are available by simply hitting the tab key after you type 'eh$'.  In this way you can explore for yourself what kinds of data are present in the hub right from the command line. This interface also allows you to access the hub programmatically to extract data that matches a particular set of criteria.

Another valuable types of metadata to pay attention to is the rdataclass.

```{r rdataclass}
head(unique(eh$rdataclass))
```

The rdataclass allows you to see which kinds of R objects the hub will return to you.  This kind of information is valuable both as a means to filter results and also as a means to explore and learn about some of the kinds of `ExperimentHub` objects that are widely available for the project.  Right now this is a pretty short list, but over time it should grow as we support more of the different kinds of `ExperimentHub` objects via the hub.

Now let's try getting the data files associated with the `r
Biocpkg("alpineData")` package using the query method. The query method lets you
search rows for specific strings, returning an `ExperimentHub` instance with
just the rows matching the query. The `ExperimentHub::package()` function
indicates the package responsible for original preparation and documentation of
the ExperimentHub data, and is a useful resource to find out more about the
data. This package may also depend, import or suggest other packages required
for manipulating the data, and for this reason the package may need to be
installed and loaded to use the resource. The package is also available in the
(cryptically named) `preparerclass` column of the metadata of the ExperimentHub
object.

One can get chain files for Drosophila melanogaster from UCSC with:

```{r alpine}
apData <- query(eh, "alpineData")
apData
```
Query has worked and you can now see that the only data present is provided by
the `r Biocpkg("alpineData")` package.

The metadata underlying this hub object can be retrieved by you

```{r show2}
apData$preparerclass
df <- mcols(apData)
```

By default the show method will only display the first 5 and last 5  rows.
There are hundreds of records present in the hub.

```{r length}
length(eh)
```

Let's look at another example, where we pull down only data
from the hub for species "mus musculus".

```{r subset}
mm <- query(eh, "mus musculus")
mm
```

We can also look at the `ExperimentHub` object in a browser using the
`display()` function. We can then filter the `ExperimentHub` object
using the Global search field on the top right corner of the page or the in-column search fields.

```{r display, eval=FALSE}
d <- display(eh)
```

# Using `ExperimentHub` to retrieve data

Looking back at our alpineData file example, if we are interested in the first file, we can get its metadata using

```{r dm2}
apData
apData["EH166"]
```

We can download the file using

```{r dm3}
apData[["EH166"]]
```

Each file is retrieved from the ExperimentHub server and the file is
also cached locally, so that the next time you need to retrieve it,
it should download much more quickly.

# Configuring `ExperimentHub` objects

When you create the `ExperimentHub` object, it will set up the object
for you with some default settings.  See `?ExperimentHub` for ways to
customize the hub source, the local cache, and other instance-specific
options, and `?getExperimentHubOption` to get or set package-global
options for use across sessions.

If you look at the object you will see some helpful information about
it such as where the data is cached and where online the hub server is
set to.

```{r show-2}
eh
```

By default the `ExperimentHub` object is set to the latest
`snapshotData` and a snapshot version that matches the version of
_Bioconductor_ that you are using. You can also learn about these data
with the appropriate methods.

```{r snapshot}
snapshotDate(eh)
```

If you are interested in using an older version of a snapshot, you can
list previous versions with the `possibleDates()` like this:

```{r possibleDates}
pd <- possibleDates(eh)
pd
```

Set the dates like this:

```{r setdate, eval=FALSE}
snapshotDate(ah) <- pd[1]
```



# Creating an ExperimentHub Package or Converting to an ExperimentHub Package

Please see `r Biocpkg("HubPub")` Vignette "CreateAHubPackage".
```
vignette("CreateAHubPackage", package="HubPub")
```

# Troubleshooting

Please see `r Biocpkg("AnnotationHub")` vignette "TroubleshootingTheCache".
```
vignette("TroubleshootingTheCache", package=AnnotationHub)
```



# Group Hub/Cache Access

The situation may occur where a hub is desired to be shared across multiple
users on a system.  This presents permissions errors.  To allow access to
multiple users create a group that the users belong to and that the underlying
hub cache belongs too. Permissions of potentially two files need to be altered depending
on what you would like individuals to be able to accomplish with the hub. A
read-only hub will require manual manipulatios of the file
BiocFileCache.sqlite.LOCK so that the group permissions are `g+rw`. To allow
users to download files to the shared hub, both the
BiocFileCache.sqlite.LOCK file and the BiocFileCache.sqlite file will need group
permissions to `g+rw`. Please google how to create a user group for your system
of interest. To find the location of the hub cache to be able to change the group
and file permissions, you may run the following in R `eh = ExperimentHub();
hubCache(eh)`. For quick reference in linux you will use `chown
currentuser:newgroup` to change the group and `chmod` to change the file
permissions: `chmod 660` or `chmod g+rw` should accomplish the correct
permissions.



# Default Caching Location Update


As of ExperimentHub version > 1.17.2, the default caching location has
changed. The default cache is now controlled by the function `tools::R_user_dir`
instead of `rappdirs::user_cache_dir`.  Users who have utilized the default
ExperimentHub location, to continue using the created cache, must move the cache and its
files to the new default location,  create a system environment variable to
point to the old location, or delete and start a new cache.

## Option 1: Move cache and files to new location

The following steps can be used to move the files to the new location:

1. Determine the old location by running the following in R
   `rappdirs::user_cache_dir(appname="ExperimentHub")`

2. Determine the new location by running the following in R
   `tools::R_user_dir("ExperimentHub", which="cache")`

3. Move the files to the new location. You can do this manually or do the
following steps in R. Remember if you have a lot of cached files, this may take
awhile.

```{r, eval=FALSE}

       # make sure you have permissions on the cache/files
       # use at own risk


       moveFiles<-function(package){
	   olddir <- path.expand(rappdirs::user_cache_dir(appname=package))
	   newdir <- tools::R_user_dir(package, which="cache")
	   dir.create(path=newdir, recursive=TRUE)
	   files <- list.files(olddir, full.names =TRUE)
	   moveres <- vapply(files,
	       FUN=function(fl){
		   filename = basename(fl)
		   newname = file.path(newdir, filename)
		   file.rename(fl, newname)
	       },
	       FUN.VALUE = logical(1))
	   if(all(moveres)) unlink(olddir, recursive=TRUE)
       }


       package="ExperimentHub"
       moveFiles(package)

```

## Option 2:  Create a system environment variable

A user may set the system environment variable `EXPERIMENT_HUB_CACHE` to control
the default location of the cache. Setting system environment variables can vary
depending on the operating system; we suggest using google to find appropriate
instructions per your operating system.

You will want to set the variable to the results of running the following in R:

```{r, eval=FALSE}
path.expand(rappdirs::user_cache_dir(appname="ExperimentHub"))
```

**NOTE:** R has `Sys.setenv` however that will only set the variable for that R
  session. It would not be available or recognized in future R sessions. It is
  important to set the variable as a global user-wide or system-wide
  environment variable so it persists in all future logins to your system.

## Option 3: Delete the old cache

Lastly, if a user does not care about the already existing default cache, the
old location may be deleted to move forward with the new default location. This
option should be used with caution. Once deleted, old cached resources will no
longer be available and have to be re-downloaded.

One can do this manually by navigating to the location indicated in the ERROR
message as `Problematic cache:` and deleting the folder and all its content.


The following can be done to delete through R code:

**CAUTION** This will remove the old cache and all downloaded resources. All
  resources will have to be re-downloaded after executing this code.

```{r, eval=FALSE}
library(ExperimentHub)
oldcache = path.expand(rappdirs::user_cache_dir(appname="ExperimentHub"))
setExperimentHubOption("CACHE", oldcache)
eh = ExperimentHub(localHub=TRUE)

## removes old location and all resources
removeCache(eh, ask=FALSE)

## create the new default caching location
newcache = tools::R_user_dir("ExperimentHub", which="cache")
setExperimentHubOption("CACHE", newcache)
eh = ExperimentHub()
```

# Session info

```{r sessionInfo}
sessionInfo()
```

\name{getExperimentHubOption}

\alias{getExperimentHubOption}
\alias{setExperimentHubOption}

\title{Get and set options for default ExperimentHub behavior.}

\description{
  These functions get or set options for creation of new
  \sQuote{ExperimentHub} instances.
}

\usage{
getExperimentHubOption(arg)
setExperimentHubOption(arg, value)
}

\arguments{
  \item{arg}{The character(1) hub options to set. see \sQuote{Details}
    for current options.}
  \item{value}{The value to be assigned to the hub option.}
}

\details{
  Supported options include:

  \describe{

    \item{\dQuote{URL}:}{character(1). The base URL of the ExperimentHub.
      Default: \url{https://experimenthub.bioconductor.org}}

    \item{\dQuote{CACHE}:}{character(1). The location of the hub
      cache. Default: \dQuote{.ExperimentHub} in the user home
      directory.}

    \item{\dQuote{MAX_DOWNLOADS}:}{numeric(1). The integer number of
      downloads allowed before triggering an error. This is to help
      avoid accidental download of a large number of ExperimentHub
      members.}

    \item{\dQuote{PROXY}:}{\code{request} object returned by
      \code{httr::use_proxy()}. The \code{request} object describes a proxy
      connection allowing Internet access, usually through a restrictive
      firewall. Setting this option sends all ExperimentHub requests through
      the proxy. Default: NULL.

      In \code{setExperimentHubOption("PROXY", value)}, \code{value} can be one of NULL,
      a \code{request} object returned by \code{httr::use_proxy()}, or a
      well-formed URL as character(1). The URL can be completely
      specified by \code{http://username:password@proxy.dom.com:8080};
      \code{username:password} and port (e.g. \code{:8080}) are
      optional. If behind a proxy it will also be useful to set the
      \code{httr::set_config(proxy)} with the proxy information.}

    \item{\dQuote{LOCAL}:}{logical(1). Should the
      ExperimentHub create a hub consisting only of previously
      downloaded resources. Default: FALSE.}

    \item{\dQuote{ASK}:}{logical(1). TRUE/FALSE should the ExperimentHub
      ask if the hub location should be created. If FALSE, the default
      location will be used and created if it doesn't exist without
      asking. If TRUE will ask the user and if in a non interactive
      session utilize a temporary directoy for the caching.
      Default: TRUE.}
  }

  Default values may also be determined by system and global R
  environment variables visible \emph{before} the package is loaded. Use
  options or variables preceded by \dQuote{EXPERIMENT_HUB_}, e.g.,
  \code{options(EXPERIMENT_HUB_MAX_DOWNLOADS=10)} prior to package load
  sets the default number of downloads to 10. 

}

\value{The requested or successfully set option.}

\author{Bioconductor Core Team}

\examples{
getExperimentHubOption("URL")
\dontrun{
setExperimentHubOption("CACHE", "~/.myHub")
}
}
\keyword{ manip }

\name{getExperimentHubOption}

\alias{getExperimentHubOption}
\alias{setExperimentHubOption}

\title{Get and set options for default ExperimentHub behavior.}

\description{
  These functions get or set options for creation of new
  \sQuote{ExperimentHub} instances.
}

\usage{
getExperimentHubOption(arg)
setExperimentHubOption(arg, value)
}

\arguments{
  \item{arg}{The character(1) hub options to set. see \sQuote{Details}
    for current options.}
  \item{value}{The value to be assigned to the hub option.}
}

\details{
  Supported options include:

  \describe{

    \item{\dQuote{URL}:}{character(1). The base URL of the ExperimentHub.
      Default: \url{https://experimenthub.bioconductor.org}}

    \item{\dQuote{CACHE}:}{character(1). The location of the hub
      cache. Default: \dQuote{.ExperimentHub} in the user home
      directory.}

    \item{\dQuote{MAX_DOWNLOADS}:}{numeric(1). The integer number of
      downloads allowed before triggering an error. This is to help
      avoid accidental download of a large number of ExperimentHub
      members.}

    \item{\dQuote{PROXY}:}{Describes a proxy
      connection allowing Internet access, usually through a restrictive
      firewall. Setting this option sends all ExperimentHub requests through
      the proxy. Default: NULL.

      In \code{setExperimentHubOption("PROXY", value)},  \code{value} can
      be NULL or a well-formed URL as character(1). The URL can be completely
      specified by \code{http://username:password@proxy.dom.com:8080};
      \code{username:password} and port (e.g. \code{:8080}) are
      optional.

      Unfortunately unlike the previously used
      `httr::set_config`, there is no option to globally set the proxy
      for httr2 requests. To get around this you can also set a system
      wide environment variable "EXPERIMENT_HUB_PROXY" for ExperimentHub
      proxy or "HUB_PROXY" that will work across all Hub classes
      (e.g. AnnotationHub and ExperimentHub)}

    \item{\dQuote{LOCAL}:}{logical(1). Should the
      ExperimentHub create a hub consisting only of previously
      downloaded resources. Default: FALSE.}

    \item{\dQuote{ASK}:}{logical(1). TRUE/FALSE should the ExperimentHub
      ask if the hub location should be created. If FALSE, the default
      location will be used and created if it doesn't exist without
      asking. If TRUE will ask the user and if in a non interactive
      session utilize a temporary directoy for the caching.
      Default: TRUE.}
  }

  Default values may also be determined by system and global R
  environment variables visible \emph{before} the package is loaded. Use
  options or variables preceded by \dQuote{EXPERIMENT_HUB_}, e.g.,
  \code{options(EXPERIMENT_HUB_MAX_DOWNLOADS=10)} prior to package load
  sets the default number of downloads to 10. 

}

\value{The requested or successfully set option.}

\author{Bioconductor Core Team}

\examples{
getExperimentHubOption("URL")
\dontrun{
setExperimentHubOption("CACHE", "~/.myHub")
}
}
\keyword{ manip }

\name{ExperimentHub-objects}
\docType{class}

% Classes
\alias{class:ExperimentHub}
\alias{ExperimentHub-class}
\alias{refreshHub}

% Constructor
\alias{ExperimentHub}

% Accessor-like methods
\alias{cache,ExperimentHub-method}
\alias{package,ExperimentHub-method}
\alias{recordStatus}

% Subsetting
\alias{[[,ExperimentHub,character,missing-method}
\alias{[[,ExperimentHub,numeric,missing-method}

% Other
\alias{listResources,ExperimentHub-method}
\alias{loadResources,ExperimentHub-method}
\alias{createHubAccessors}

\title{ExperimentHub objects and their related methods and functions}

\description{

  Use \code{ExperimentHub} to interact with Bioconductor's ExperimentHub
  service.  Query the instance to discover and use resources that are of
  interest, and then easily download and import the resource into R for
  immediate use.

  Use \code{ExperimentHub()} to retrieve information about all records
  in the hub. If working offline, add argument \code{localHub=TRUE} to
  work with a local, non-updated hub; It will only have resources
  available that have previously been downloaded. If offline, Please
  also see BiocManager vignette section on offline use to ensure proper
  funcionality. To force redownload of the hub,
  \code{refreshHub(hubClass="ExperimentHub")} can be utilized.

  Discover records in a hub using \code{mcols()}, \code{query()},
  \code{subset()}, \code{[}, and \code{display()}.

  Retrieve individual records using \code{[[}. On first use of a
  resource, the corresponding files or other hub resources are
  downloaded from the internet to a local cache. On this and all
  subsequent uses the files are quickly input from the cache into the R
  session. If a user wants to download the file again and not use the
  cache version add the argument \code{force=TRUE}.

  \code{ExperimentHub} records can be added (and sometimes removed) at
  any time. \code{snapshotDate()} restricts hub records to those
  available at the time of the snapshot. \code{possibleDates()} lists
  snapshot dates valid for the current version of Bioconductor.

  The location of the local cache can be found (and updated) with
  \code{getExperimentHubOption} and \code{setExperimentHubOption};
  \code{removeCache} removes all cache resources.


  For common hub troubleshooting, please see the AnnotationHub vignette
  entitled `vignette("TroubleshootingTheCache", package=AnnotationHub)`.
  
}

\section{Constructors}{
  \describe{
    \item{}{
      \code{ExperimentHub(..., hub=getExperimentHubOption("URL"),
	cache=getExperimentHubOption("CACHE"),
	proxy=getExperimentHubOption("PROXY"),
	localHub=getExperimentHubOption("LOCAL"))}:

      Create an \code{ExperimentHub} instance, possibly updating the
      current database of records.

      See \code{getExperimentHubOption} for more details on constructor
      arguments. The ellipse is current not utilized; arguments are
      passed into the virtual Hub class constructor and it is needed as
      a placeholder.
    }
  }
}

\section{Accessors}{
  In the code snippets below, \code{x} and \code{object} are
  ExperimentHub objects.

  \describe{
    \item{}{
      \code{hubCache(x)}:
      Gets the file system location of the local ExperimentHub cache.
    }
    \item{}{
      \code{hubUrl(x)}:
      Gets the URL for the online hub.
    }
    \item{}{
      \code{length(x)}:
      Get the number of hub records.
    }
    \item{}{
      \code{names(x)}:
      Get the names (ExperimentHub unique identifiers, of the form
      AH12345) of the hub records.
    }
    \item{}{
      \code{fileName(x)}:
      Get the file path of the hub records as stored in the local cache
      (ExperimentHub files are stored as unique numbers, of the form
      12345).  NA is returned for those records which have not been
      cached.
    }
    \item{}{
      \code{package(x)}:
      Returns a named character vector of package name associated with
      the hub resource.
    }
    \item{}{
      \code{recordStatus(x, record)}:
      Returns a \code{data.frame} of the record id and status. \code{x} must
      be a \code{Hub} object and \code{record} must be a \code{character(1)}.
      Can be used to discover why a resource was removed from the hub.
    }
    \item{}{
      \code{mcols(x)}:
      Get the metadata columns describing each record. Columns include:
      \describe{

	\item{title}{Record title, frequently the file name of the
	  object.}

	\item{dataprovider}{Original provider of the resource, e.g.,
	  Ensembl, UCSC.}

	\item{species}{The species for which the record is most
	  relevant, e.g., \sQuote{Homo sapiens}.}

	\item{taxonomyid}{NCBI taxonomy identifier of the species.}

	\item{genome}{Genome build relevant to the record, e.g., hg19.}

	\item{description}{Textual description of the resource,
	  frequently automatically generated from file path and other
	  information available when the record was created.}

	\item{tags}{Single words added to the record to facilitate
	  identification, e.g,. TCGA, Roadmap.}

	\item{rdataclass}{The class of the R object used to represent
	  the object when imported into R, e.g., \code{GRanges},
	  \code{VCFFile}.}

	\item{sourceurl}{Original URL of the resource.}

	\item{sourectype}{Format of the original resource, e.g., BED
	  file.}
      }
    }
  }
}

\section{Subsetting and related operations}{
  In the code snippets below, \code{x} is an ExperimentHub object.

  \describe{
    \item{}{
      \code{x$name}:
      Convenient reference to individual metadata columns, e.g.,
      \code{x$species}.
    }
    \item{}{
      \code{x[i]}:
      Numerical, logical, or character vector (of ExperimentHub names)
      to subset the hub, e.g., \code{x[x$species == "Homo sapiens"]}.
    }
    \item{}{
      \code{x[[i, force=FALSE, verbose=TRUE]]}:
      Numerical or character scalar to retrieve (if necessary) and
      import the resource into R. If a user wants to download the file
      again and not use the cache version add the argument
      \code{force=TRUE}. \code{verbose=FALSE} will quiet status messages.
    }
    \item{}{
      \code{query(x, pattern, ignore.case=TRUE, pattern.op= `&`)}:
      Return an ExperimentHub subset containing only those elements
      whose metadata matches \code{pattern}. Matching uses
      \code{pattern} as in \code{\link{grepl}} to search the
      \code{as.character} representation of each column, performing a
      logical \code{`&`} across columns.
      e.g., \code{query(x, c("Homo sapiens", "hg19", "GTF"))}.
      \describe{
	\item{\code{pattern}}{A character vector of patterns to search
	  (via \code{grepl}) for in any of the \code{mcols()} columns.}
	\item{\code{ignore.case}}{A logical(1) vector indicating whether
	  the search should ignore case (TRUE) or not (FALSE).}
	\item{\code{pattern.op}}{Any function of two arguments,
	  describing how matches across pattern elements are to be
	  combined. The default \code{`&`} requires that only records
	  with \emph{all} elements of \code{pattern} in their metadata
	  columns are returned. \code{`&`}, \code{`|`} and \code{`!`}
	  are most notably available. See \code{"?&"} or
	  \code{?base::Ops} for more information.}
      }
    }
    \item{}{
      \code{subset(x, subset)}:
      Return the subset of records containing only those elements whose
      metadata satisfies the \emph{expression} in \code{subset}. The
      expression can reference columns of \code{mcols(x)}, and should
      return a logical vector of length \code{length(x)}.
      e.g., \code{subset(x, species == "Homo sapiens" &
	genome=="GRCh38")}.
    }
    \item{}{
      \code{display(object)}:
      Open a web browser allowing for easy selection of hub records via
      interactive tabular display. Return value is the subset of hub
      records identified while navigating the display.
    }
    \item{}{
      \code{listResources(hub, package, filterBy=character())}:
      List resources in ExperimentHub associated with \code{package}.
      \code{filterBy} is a character vector of search terms.
    }
    \item{}{
      \code{loadResources(hub, package, filterBy=character())}:
      Load resources in ExperimentHub associated with \code{package}.
      \code{filterBy} is a character vector of search terms.
    }
  }
}

\section{Cache and hub management}{
  In the code snippets below, \code{x} is an ExperimentHub object.
  \describe{
    \item{}{
      \code{snapshotDate(x)} and \code{snapshotDate(x) <- value}:
      Gets or sets the date for the snapshot in use. \code{value} should
      be one of \code{possibleDates()}.
    }
    \item{}{
      \code{possibleDates(x)}:
      Lists the valid snapshot dates for the version of Bioconductor that
      is being run (e.g., BiocManager::version()).
    }
    \item{}{
      \code{cache(x)} and \code{cache(x) <- NULL}: Adds (downloads) all
      resources in \code{x}, or removes all local resources
      corresponding to the records in \code{x} from the cache. In this case,
      \code{x} would typically be a small subset of ExperimentHub
      resources. If \code{x} is a subset hub from a larger hub, and
      \code{localHub=TRUE} was used to construct the hubs, 
      the original object will need to be reconstructed to reflect the
      removed resources. See also \code{removeResources} for a nicer interface
      for removing cached resources, or \code{removeCache} for deleting the hub
      cache entirely.
    }
    \item{}{
      \code{hubUrl(x)}:
      Gets the URL for the online ExperimentHub.
    }
    \item{}{
      \code{hubCache(x)}:
      Gets the file system location of the local ExperimentHub cache.
    }
    \item{}{
      \code{refreshHub(..., hub, cache, proxy,
        hubClass=c("AnnotationHub", "ExperimentHub"))}:
      Force redownload of Hub sqlite file. This returns a Hub object as
      if calling the constructor (ie. ExperimentHub()). For force
      redownload specifically for ExperimentHub the base call should be
      \code{refreshHub(hubClass="ExperimentHub")}
    }   
    \item{}{
      \code{removeResources(hub, ids)}:
      Removes listed ids from the local cache. ids are "EH" ids. Returns
      an updated hub object. To work with updated hub object suggested
      syntax is to reassign (ie. \code{hub = removeResources(hub,
      "EH1")}). If ids are missing will remove all previously downloaded
      local resources.
    }
    \item{}{
      \code{removeCache(x)}:
      Removes local ExperimentHub database and all related resources. After
      calling this function, the user will have to download any ExperimentHub
      resources again.
    }
  }
}

\section{Coercion}{
  In the code snippets below, \code{x} is an ExperimentHub object.
  \describe{
    \item{}{
      \code{as.list(x)}:
      Coerce x to a list of hub instances, one entry per
      element. Primarily for internal use.
    }
    \item{}{
      \code{c(x, ...)}:
      Concatenate one or more sub-hub. Sub-hubs must reference the same
      ExperimentHub instance. Duplicate entries are removed.
    }
  }
}

\section{For developers}{
  \code{createHubAccessors(pkgname, titles): } This helper is intended to be
  used by ExperimentHub package developers in their .onLoad() function in
  zzz.R. It generates and exports functions by resource name which allows the
  resource to be loaded with resource123() in addition to the standard method
  via the ExperimentHub interface, e.g., ehub[["EH123"]]. When 'metadata=TRUE'
  just the metadata are displayed, when 'metadata=FALSE' the full resource is
  loaded, e.g, resource123(metadata=TRUE). See vignette for more information.
}

\author{Bioconductor Core Team}

\examples{
  ## Create an ExperimentHub object
  ehub <- ExperimentHub()
  ehub

  ## Display packages associated with resources
  unique(package(ehub))
  query(ehub, "alpineData")

  ## Search metadata by general terms
  query(ehub, c("FASTQ", "Homo sapiens"))
}

\keyword{classes}
\keyword{methods}

\name{ExperimentHub-objects}
\docType{class}

% Classes
\alias{class:ExperimentHub}
\alias{ExperimentHub-class}
\alias{refreshHub}

% Constructor
\alias{ExperimentHub}

% Accessor-like methods
\alias{cache,ExperimentHub-method}
\alias{package,ExperimentHub-method}
\alias{recordStatus}

% Subsetting
\alias{[[,ExperimentHub,character,missing-method}
\alias{[[,ExperimentHub,numeric,missing-method}

% Other
\alias{listResources,ExperimentHub-method}
\alias{loadResources,ExperimentHub-method}
\alias{createHubAccessors}

\title{ExperimentHub objects and their related methods and functions}

\description{

  Use \code{ExperimentHub} to interact with Bioconductor's ExperimentHub
  service.  Query the instance to discover and use resources that are of
  interest, and then easily download and import the resource into R for
  immediate use.

  Use \code{ExperimentHub()} to retrieve information about all records
  in the hub. If working offline, add argument \code{localHub=TRUE} to
  work with a local, non-updated hub; It will only have resources
  available that have previously been downloaded. If offline, Please
  also see BiocManager vignette section on offline use to ensure proper
  funcionality. To force redownload of the hub,
  \code{refreshHub(hubClass="ExperimentHub")} can be utilized.

  If you are operating behind a proxy please see the ExperimentHub
  Vignette section on "Access behind a Proxy" for setting up
  configuration to allow ExperimentHub to run properly.
    
  Discover records in a hub using \code{mcols()}, \code{query()},
  \code{subset()}, and \code{[}.

  Retrieve individual records using \code{[[}. On first use of a
  resource, the corresponding files or other hub resources are
  downloaded from the internet to a local cache. On this and all
  subsequent uses the files are quickly input from the cache into the R
  session. If a user wants to download the file again and not use the
  cache version add the argument \code{force=TRUE}.

  \code{ExperimentHub} records can be added (and sometimes removed) at
  any time. \code{snapshotDate()} restricts hub records to those
  available at the time of the snapshot. \code{possibleDates()} lists
  snapshot dates valid for the current version of Bioconductor.

  The location of the local cache can be found (and updated) with
  \code{getExperimentHubOption} and \code{setExperimentHubOption};
  \code{removeCache} removes all cache resources.


  For common hub troubleshooting, please see the AnnotationHub vignette
  entitled `vignette("TroubleshootingTheHubs", package="AnnotationHub")`.
  
}

\section{Constructors}{
  \describe{
    \item{}{
      \code{ExperimentHub(..., hub=getExperimentHubOption("URL"),
	cache=getExperimentHubOption("CACHE"),
	proxy=getExperimentHubOption("PROXY"),
	localHub=getExperimentHubOption("LOCAL"))}:

      Create an \code{ExperimentHub} instance, possibly updating the
      current database of records.

      See \code{getExperimentHubOption} for more details on constructor
      arguments. The ellipse is current not utilized; arguments are
      passed into the virtual Hub class constructor and it is needed as
      a placeholder.
    }
  }
}

\section{Accessors}{
  In the code snippets below, \code{x} and \code{object} are
  ExperimentHub objects.

  \describe{
    \item{}{
      \code{hubCache(x)}:
      Gets the file system location of the local ExperimentHub cache.
    }
    \item{}{
      \code{hubUrl(x)}:
      Gets the URL for the online hub.
    }
    \item{}{
      \code{length(x)}:
      Get the number of hub records.
    }
    \item{}{
      \code{names(x)}:
      Get the names (ExperimentHub unique identifiers, of the form
      AH12345) of the hub records.
    }
    \item{}{
      \code{fileName(x)}:
      Get the file path of the hub records as stored in the local cache
      (ExperimentHub files are stored as unique numbers, of the form
      12345).  NA is returned for those records which have not been
      cached.
    }
    \item{}{
      \code{package(x)}:
      Returns a named character vector of package name associated with
      the hub resource.
    }
    \item{}{
      \code{recordStatus(x, record)}:
      Returns a \code{data.frame} of the record id and status. \code{x} must
      be a \code{Hub} object and \code{record} must be a \code{character(1)}.
      Can be used to discover why a resource was removed from the hub.
    }
    \item{}{
      \code{mcols(x)}:
      Get the metadata columns describing each record. Columns include:
      \describe{

	\item{title}{Record title, frequently the file name of the
	  object.}

	\item{dataprovider}{Original provider of the resource, e.g.,
	  Ensembl, UCSC.}

	\item{species}{The species for which the record is most
	  relevant, e.g., \sQuote{Homo sapiens}.}

	\item{taxonomyid}{NCBI taxonomy identifier of the species.}

	\item{genome}{Genome build relevant to the record, e.g., hg19.}

	\item{description}{Textual description of the resource,
	  frequently automatically generated from file path and other
	  information available when the record was created.}

	\item{tags}{Single words added to the record to facilitate
	  identification, e.g,. TCGA, Roadmap.}

	\item{rdataclass}{The class of the R object used to represent
	  the object when imported into R, e.g., \code{GRanges},
	  \code{VCFFile}.}

	\item{sourceurl}{Original URL of the resource.}

	\item{sourectype}{Format of the original resource, e.g., BED
	  file.}
      }
    }
  }
}

\section{Subsetting and related operations}{
  In the code snippets below, \code{x} is an ExperimentHub object.

  \describe{
    \item{}{
      \code{x$name}:
      Convenient reference to individual metadata columns, e.g.,
      \code{x$species}.
    }
    \item{}{
      \code{x[i]}:
      Numerical, logical, or character vector (of ExperimentHub names)
      to subset the hub, e.g., \code{x[x$species == "Homo sapiens"]}.
    }
    \item{}{
      \code{x[[i, force=FALSE, verbose=TRUE, config=list(), progress=TRUE]]}:
      Numerical or character scalar to retrieve (if necessary) and
      import the resource into R. If a user wants to download the file
      again and not use the cache version add the argument
      \code{force=TRUE}. \code{verbose=FALSE} will quiet status
      messages. \code{config=list()} passes arguments to
      \code{httr2::req_options}. The names of the items should be valid
      curl options as defined in \code{curl::curl_options}. A progress
      bar for longer downloads is active by default but can be controlled
      with the argument \code{progress=TRUE}.
    }
    \item{}{
      \code{query(x, pattern, ignore.case=TRUE, pattern.op= `&`)}:
      Return an ExperimentHub subset containing only those elements
      whose metadata matches \code{pattern}. Matching uses
      \code{pattern} as in \code{\link{grepl}} to search the
      \code{as.character} representation of each column, performing a
      logical \code{`&`} across columns.
      e.g., \code{query(x, c("Homo sapiens", "hg19", "GTF"))}.
      \describe{
	\item{\code{pattern}}{A character vector of patterns to search
	  (via \code{grepl}) for in any of the \code{mcols()} columns.}
	\item{\code{ignore.case}}{A logical(1) vector indicating whether
	  the search should ignore case (TRUE) or not (FALSE).}
	\item{\code{pattern.op}}{Any function of two arguments,
	  describing how matches across pattern elements are to be
	  combined. The default \code{`&`} requires that only records
	  with \emph{all} elements of \code{pattern} in their metadata
	  columns are returned. \code{`&`}, \code{`|`} and \code{`!`}
	  are most notably available. See \code{"?&"} or
	  \code{?base::Ops} for more information.}
      }
    }
    \item{}{
      \code{subset(x, subset)}:
      Return the subset of records containing only those elements whose
      metadata satisfies the \emph{expression} in \code{subset}. The
      expression can reference columns of \code{mcols(x)}, and should
      return a logical vector of length \code{length(x)}.
      e.g., \code{subset(x, species == "Homo sapiens" &
	genome=="GRCh38")}.
    }
    \item{}{
      \code{listResources(hub, package, filterBy=character())}:
      List resources in ExperimentHub associated with \code{package}.
      \code{filterBy} is a character vector of search terms.
    }
    \item{}{
      \code{loadResources(hub, package, filterBy=character())}:
      Load resources in ExperimentHub associated with \code{package}.
      \code{filterBy} is a character vector of search terms.
    }
  }
}

\section{Cache and hub management}{
  In the code snippets below, \code{x} is an ExperimentHub object.
  \describe{
    \item{}{
      \code{snapshotDate(x)} and \code{snapshotDate(x) <- value}:
      Gets or sets the date for the snapshot in use. \code{value} should
      be one of \code{possibleDates()}.
    }
    \item{}{
      \code{possibleDates(x)}:
      Lists the valid snapshot dates for the version of Bioconductor that
      is being run (e.g., BiocManager::version()).
    }
    \item{}{
      \code{cache(x)} and \code{cache(x) <- NULL}: Adds (downloads) all
      resources in \code{x}, or removes all local resources
      corresponding to the records in \code{x} from the cache. In this case,
      \code{x} would typically be a small subset of ExperimentHub
      resources. If \code{x} is a subset hub from a larger hub, and
      \code{localHub=TRUE} was used to construct the hubs, 
      the original object will need to be reconstructed to reflect the
      removed resources. See also \code{removeResources} for a nicer interface
      for removing cached resources, or \code{removeCache} for deleting the hub
      cache entirely. See also additional arguments from \code{[[}.
    }
    \item{}{
      \code{hubUrl(x)}:
      Gets the URL for the online ExperimentHub.
    }
    \item{}{
      \code{hubCache(x)}:
      Gets the file system location of the local ExperimentHub cache.
    }
    \item{}{
      \code{refreshHub(..., hub, cache, proxy,
        hubClass=c("AnnotationHub", "ExperimentHub"))}:
      Force redownload of Hub sqlite file. This returns a Hub object as
      if calling the constructor (ie. ExperimentHub()). For force
      redownload specifically for ExperimentHub the base call should be
      \code{refreshHub(hubClass="ExperimentHub")}
    }   
    \item{}{
      \code{removeResources(hub, ids)}:
      Removes listed ids from the local cache. ids are "EH" ids. Returns
      an updated hub object. To work with updated hub object suggested
      syntax is to reassign (ie. \code{hub = removeResources(hub,
      "EH1")}). If ids are missing will remove all previously downloaded
      local resources.
    }
    \item{}{
      \code{removeCache(x)}:
      Removes local ExperimentHub database and all related resources. After
      calling this function, the user will have to download any ExperimentHub
      resources again.
    }
  }
}

\section{Coercion}{
  In the code snippets below, \code{x} is an ExperimentHub object.
  \describe{
    \item{}{
      \code{as.list(x)}:
      Coerce x to a list of hub instances, one entry per
      element. Primarily for internal use.
    }
    \item{}{
      \code{c(x, ...)}:
      Concatenate one or more sub-hub. Sub-hubs must reference the same
      ExperimentHub instance. Duplicate entries are removed.
    }
  }
}

\section{For developers}{
  \code{createHubAccessors(pkgname, titles): } This helper is intended to be
  used by ExperimentHub package developers in their .onLoad() function in
  zzz.R. It generates and exports functions by resource name which allows the
  resource to be loaded with resource123() in addition to the standard method
  via the ExperimentHub interface, e.g., ehub[["EH123"]]. When 'metadata=TRUE'
  just the metadata are displayed, when 'metadata=FALSE' the full resource is
  loaded, e.g, resource123(metadata=TRUE). See vignette for more information.
}

\author{Bioconductor Core Team}

\examples{
  ## Create an ExperimentHub object
  ehub <- ExperimentHub()
  ehub

  ## Display packages associated with resources
  unique(package(ehub))
  query(ehub, "alpineData")

  ## Search metadata by general terms
  query(ehub, c("FASTQ", "Homo sapiens"))
}

\keyword{classes}
\keyword{methods}

\name{utilities}
\alias{utilities}

\alias{listResources}
\alias{listResources,AnnotationHub-method}
\alias{loadResources}
\alias{loadResources,AnnotationHub-method}


\title{
  Utility functions for discovering package-specific Hub resources.
}

\description{
  List and load resources from ExperimentHub filtered by package
  name and optional search terms.
}

\details{
  Currently \code{listResources} and \code{loadResources} are only meaningful
  for \code{ExperimentHub} objects.
}

\usage{
listResources(hub, package, filterBy = character())
loadResources(hub, package, filterBy = character())
}

\arguments{
  \item{hub}{
    A \code{Hub} object; currently only meaningful for \code{ExperimentHub}.
  }
  \item{package}{
    A \code{character(1)} name of a package with resources hosted in the Hub.
  }
  \item{filterBy}{
    A \code{character()} vector of search terms for additional filtering.
    Can be any terms found in the metadata (mcols()) of the resources.
    When not provided, there is no additional filtering and all resources
    associated with the given package are returned.
  }
}

\value{
  \code{listResources} returns a character vector;
  \code{loadResources} returns a list of data objects.
}

\examples{
## Packages with resources hosted in ExperimentHub:
library(ExperimentHub)
eh <- ExperimentHub()
head(
    unique(package(eh))
)

## All resources associated with the 'GSE62944' package:
listResources(eh, "GSE62944")

## Resources associated with the 'curatedMetagenomicData' package
## filtered by 'plaque.abundance':
listResources(eh, "curatedMetagenomicData", "plaque")

\dontrun{
## 'loadResources()' returns a list of the data objects:
loadResources(eh, "curatedMetagenomicData", "plaque")
}
}
\keyword{utilities}

\name{utilities}
\alias{utilities}

\alias{listResources}
\alias{listResources,AnnotationHub-method}
\alias{loadResources}
\alias{loadResources,AnnotationHub-method}


\title{
  Utility functions for discovering package-specific Hub resources.
}

\description{
  List and load resources from ExperimentHub filtered by package
  name and optional search terms.
}

\details{
  Currently \code{listResources} and \code{loadResources} are only meaningful
  for \code{ExperimentHub} objects.
}

\usage{
listResources(hub, package, filterBy = character())
loadResources(hub, package, filterBy = character())
}

\arguments{
  \item{hub}{
    A \code{Hub} object; currently only meaningful for \code{ExperimentHub}.
  }
  \item{package}{
    A \code{character(1)} name of a package with resources hosted in the Hub.
  }
  \item{filterBy}{
    A \code{character()} vector of search terms for additional filtering. 
    Can be any terms found in the metadata (mcols()) of the resources. 
    When not provided, there is no additional filtering and all resources 
    associated with the given package are returned.
  }
}

\value{
  \code{listResources} returns a character vector; 
  \code{loadResources} returns a list of data objects.
}

\examples{
\dontrun{
## Packages with resources hosted in ExperimentHub:
require(ExperimentHub)
eh <- ExperimentHub()
unique(package(eh))

## All resources associated with the 'GSE62944' package:
listResources(eh, "GSE62944")

## Resources associated with the 'curatedMetagenomicData' package
## filtered by 'plaque.abundance':
listResources(eh, "curatedMetagenomicData", "plaque.abundance")

## 'loadResources()' returns a list of the data objects:
loadResources(eh, "curatedMetagenomicData", "plaque.abundance")
}
}
\keyword{utilities}
